import pytest
from icupy import U_ICU_VERSION_MAJOR_NUM

if U_ICU_VERSION_MAJOR_NUM < 50:
    pytest.skip("ICU4C<50", allow_module_level=True)

from icupy import ListFormatter, Locale, UnicodeString


def test_api():
    fmt = ListFormatter.create_instance(Locale("en", "US"))

    # [2]
    # UnicodeString &icu::ListFormatter::format(
    #       const UnicodeString items[],
    #       int32_t n_items,
    #       UnicodeString &appendTo,
    #       UErrorCode &errorCode
    # )
    items = [
        UnicodeString("Alice"),
        UnicodeString("Bob"),
        UnicodeString("Charlie"),
        UnicodeString("Delta"),
    ]
    append_to = UnicodeString()
    result = fmt.format(items, len(items), append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "Alice, Bob, Charlie, and Delta"

    append_to.remove()
    result = fmt.format(items, -1, append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "Alice, Bob, Charlie, and Delta"


def test_create_instance():
    # [1]
    # static ListFormatter *icu::ListFormatter::createInstance(
    #       const Locale &locale,
    #       UErrorCode &errorCode
    # )
    fmt1 = ListFormatter.create_instance(Locale("en", "US"))
    assert isinstance(fmt1, ListFormatter)

    fmt1a = ListFormatter.create_instance("en-US")
    assert isinstance(fmt1a, ListFormatter)

    # [3]
    # static ListFormatter *icu::ListFormatter::createInstance(
    #       UErrorCode &errorCode
    # )
    fmt3 = ListFormatter.create_instance()
    assert isinstance(fmt3, ListFormatter)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 67, reason="ICU4C<67")
def test_create_instance_67():
    from icupy import UListFormatterType, UListFormatterWidth

    # [2]
    # static ListFormatter *icu::ListFormatter::createInstance(
    #       const Locale &locale,
    #       UListFormatterType type,
    #       UListFormatterWidth width,
    #       UErrorCode &errorCode
    # )
    fmt2 = ListFormatter.create_instance(
        Locale("en", "US"),
        UListFormatterType.ULISTFMT_TYPE_UNITS,
        UListFormatterWidth.ULISTFMT_WIDTH_SHORT)
    assert isinstance(fmt2, ListFormatter)

    fmt2a = ListFormatter.create_instance(
        "en-US",
        UListFormatterType.ULISTFMT_TYPE_UNITS,
        UListFormatterWidth.ULISTFMT_WIDTH_SHORT)
    assert isinstance(fmt2a, ListFormatter)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 64, reason="ICU4C<64")
def test_format_strings_to_value():
    from icupy import (
        ConstrainedFieldPosition, FormattedList, FormattedValue,
        UFieldCategory, UListFormatterField, UnicodeStringAppendable,
    )

    assert issubclass(FormattedList, FormattedValue)
    fmt = ListFormatter.create_instance("en")

    # FormattedList icu::ListFormatter::formatStringsToValue(
    #       const UnicodeString items[],
    #       int32_t n_items,
    #       UErrorCode &errorCode
    # )
    items = [
        UnicodeString("hello"),
        UnicodeString("wonderful"),
        UnicodeString("world"),
    ]
    fl = fmt.format_strings_to_value(items, len(items))
    assert isinstance(fl, FormattedList)

    # Appendable &icu::FormattedList::appendTo(
    #       Appendable &appendable,
    #       UErrorCode &status
    # )
    s = UnicodeString("x")
    appendable = UnicodeStringAppendable(s)
    result = fl.append_to(appendable)
    assert isinstance(result, UnicodeStringAppendable)
    assert id(result) == id(appendable)
    assert s == "xhello, wonderful, and world"

    # UBool icu::FormattedList::nextPosition(
    #       ConstrainedFieldPosition &cfpos,
    #       UErrorCode &status
    # )
    cfpos = ConstrainedFieldPosition()
    assert fl.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_LIST_SPAN
    assert cfpos.get_field() == 0
    assert (cfpos.get_start(), cfpos.get_limit()) == (0, 5)

    assert fl.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_LIST
    assert cfpos.get_field() == UListFormatterField.ULISTFMT_ELEMENT_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (0, 5)

    assert fl.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_LIST
    assert cfpos.get_field() == UListFormatterField.ULISTFMT_LITERAL_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (5, 7)

    assert fl.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_LIST_SPAN
    assert cfpos.get_field() == 1
    assert (cfpos.get_start(), cfpos.get_limit()) == (7, 16)

    assert fl.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_LIST
    assert cfpos.get_field() == UListFormatterField.ULISTFMT_ELEMENT_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (7, 16)

    assert fl.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_LIST
    assert cfpos.get_field() == UListFormatterField.ULISTFMT_LITERAL_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (16, 22)

    assert fl.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_LIST_SPAN
    assert cfpos.get_field() == 2
    assert (cfpos.get_start(), cfpos.get_limit()) == (22, 27)

    assert fl.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_LIST
    assert cfpos.get_field() == UListFormatterField.ULISTFMT_ELEMENT_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (22, 27)

    assert not fl.next_position(cfpos)

    # UnicodeString icu::FormattedList::toString(UErrorCode &status)
    result = fl.to_string()
    assert isinstance(result, UnicodeString)
    assert result == "hello, wonderful, and world"

    # UnicodeString icu::FormattedList::toTempString(UErrorCode &status)
    result = fl.to_temp_string()
    assert isinstance(result, UnicodeString)
    assert result == "hello, wonderful, and world"

    fl = fmt.format_strings_to_value(items, -1)  # n_items is optional
    assert fl.to_temp_string() == "hello, wonderful, and world"

    fl = fmt.format_strings_to_value(items)  # n_items is optional
    assert fl.to_temp_string() == "hello, wonderful, and world"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 52, reason="ICU4C<52")
def test_list_formatter_52():
    fmt = ListFormatter.create_instance(Locale("en", "US"))
    assert isinstance(fmt, ListFormatter)

    # [1]
    # icu::ListFormatter::ListFormatter(const ListFormatter &)
    _ = ListFormatter(fmt)



from icupy import (
    CollationKey, Collator, Locale, UCollationResult, UColAttribute,
    UColAttributeValue, UnicodeString
)


def test_api():
    coll = Collator.create_instance(Locale.get_english())
    coll.set_attribute(UColAttribute.UCOL_STRENGTH,
                       UColAttributeValue.UCOL_TERTIARY)
    source = UnicodeString("Abcd")

    # [1]
    # icu::CollationKey::CollationKey()
    key = CollationKey()
    coll.get_collation_key(source, key)

    key1 = CollationKey()

    # const uint8_t *icu::CollationKey::getByteArray(int32_t &count)
    values = key.get_byte_array()
    assert isinstance(values, list)
    assert len(values) > 0
    assert all(isinstance(x, int) for x in values)

    # [2]
    # icu::CollationKey::CollationKey(
    #       const uint8_t *values,
    #       int32_t count
    # )
    key2 = CollationKey(values, len(values))

    # [3]
    # icu::CollationKey::CollationKey(const CollationKey &other)
    key3 = CollationKey(key)

    # [2]
    # UCollationResult icu::CollationKey::compareTo(
    #       const CollationKey &target,
    #       UErrorCode &status
    # )
    assert key.compare_to(key1) == UCollationResult.UCOL_GREATER
    assert key.compare_to(key2) == UCollationResult.UCOL_EQUAL
    assert key.compare_to(key3) == UCollationResult.UCOL_EQUAL

    assert key1.compare_to(key) == UCollationResult.UCOL_LESS
    assert key1.compare_to(key2) == UCollationResult.UCOL_LESS
    assert key1.compare_to(key3) == UCollationResult.UCOL_LESS

    assert key2.compare_to(key) == UCollationResult.UCOL_EQUAL
    assert key2.compare_to(key1) == UCollationResult.UCOL_GREATER
    assert key2.compare_to(key3) == UCollationResult.UCOL_EQUAL

    assert key3.compare_to(key) == UCollationResult.UCOL_EQUAL
    assert key3.compare_to(key1) == UCollationResult.UCOL_GREATER
    assert key3.compare_to(key2) == UCollationResult.UCOL_EQUAL

    # int32_t icu::CollationKey::hashCode(void)
    assert key1.hash_code() != key.hash_code()
    assert key2.hash_code() == key.hash_code()
    assert key3.hash_code() == key.hash_code()

    # UBool icu::CollationKey::isBogus(void)
    assert not key1.is_bogus()
    assert not key2.is_bogus()
    assert not key3.is_bogus()

    # UBool icu::CollationKey::operator!=(const CollationKey &source)
    assert key1 != key

    # UBool icu::CollationKey::operator==(const CollationKey &source)
    assert key2 == key
    assert key3 == key



import copy

import pytest
from icupy import (
    AnnualTimeZoneRule, BasicTimeZone, DateTimeRule, GregorianCalendar,
    InitialTimeZoneRule, Locale, RuleBasedTimeZone, SimpleTimeZone,
    StringEnumeration, TimeZone, TimeZoneTransition, UCalendarDaysOfWeek,
    UCalendarMonths, USystemTimeZoneType, U_ICU_VERSION_MAJOR_NUM,
    U_MILLIS_PER_HOUR as HOUR, UnicodeString, VTimeZone,
)


def test_basic_time_zone():
    assert issubclass(BasicTimeZone, TimeZone)

    zone1 = TimeZone.create_time_zone("America/Los_Angeles")
    assert isinstance(zone1, BasicTimeZone)  # OlsonTimeZone
    assert not isinstance(zone1, RuleBasedTimeZone)
    assert not isinstance(zone1, SimpleTimeZone)
    assert not isinstance(zone1, VTimeZone)

    # BasicTimeZone *icu::BasicTimeZone::clone()
    zone1a = zone1.clone()
    assert isinstance(zone1a, BasicTimeZone)
    assert zone1a == zone1

    zone1b = copy.copy(zone1)
    assert zone1b == zone1

    zone1c = copy.deepcopy(zone1)
    assert zone1c == zone1

    # int32_t icu::BasicTimeZone::countTransitionRules(UErrorCode &status)
    assert zone1.count_transition_rules() == 4

    # UBool icu::BasicTimeZone::getNextTransition(
    #       UDate base,
    #       UBool inclusive,
    #       TimeZoneTransition &result
    # )
    base = 1230681600000.0  # 2008-12-31T00:00:00Z
    result = TimeZoneTransition()
    assert zone1.get_next_transition(base, False, result)
    assert result.get_time() == 1236506400000.0  # 2009-03-08T10:00:00Z

    # UBool icu::BasicTimeZone::getPreviousTransition(
    #       UDate base,
    #       UBool inclusive,
    #       TimeZoneTransition &result
    # )
    assert zone1.get_previous_transition(base, False, result)
    assert result.get_time() == 1225616400000.0  # 2008-11-02T09:00:00Z

    # void icu::BasicTimeZone::getSimpleRulesNear(
    #       UDate date,
    #       InitialTimeZoneRule *&initial,
    #       AnnualTimeZoneRule *&std,
    #       AnnualTimeZoneRule *&dst,
    #       UErrorCode &status
    # )
    initial, std, dst = zone1.get_simple_rules_near(0)
    assert isinstance(initial, InitialTimeZoneRule)
    assert initial.get_raw_offset() == -8 * HOUR
    assert initial.get_dst_savings() == 1 * HOUR
    assert isinstance(std, AnnualTimeZoneRule)
    assert std.get_raw_offset() == -8 * HOUR
    assert std.get_dst_savings() == 0
    dr = std.get_rule()
    assert dr.get_rule_month() == UCalendarMonths.UCAL_OCTOBER
    assert dr.get_rule_day_of_month() == 0
    assert isinstance(dst, AnnualTimeZoneRule)
    assert dst.get_raw_offset() == -8 * HOUR
    assert dst.get_dst_savings() == 1 * HOUR
    dr = dst.get_rule()
    assert dr.get_rule_month() == UCalendarMonths.UCAL_APRIL
    assert dr.get_rule_day_of_month() == 0

    # void icu::BasicTimeZone::getTimeZoneRules(
    #       const InitialTimeZoneRule *&initial,
    #       const TimeZoneRule *trsrules[],
    #       int32_t &trscount,
    #       UErrorCode &status
    # )
    initial, trsrules = zone1.get_time_zone_rules()
    assert isinstance(initial, InitialTimeZoneRule)
    assert isinstance(trsrules, list)
    assert len(trsrules) == 4
    assert any(isinstance(x, AnnualTimeZoneRule) for x in trsrules)

    # UBool icu::BasicTimeZone::hasEquivalentTransitions(
    #       const BasicTimeZone &tz,
    #       UDate start,
    #       UDate end,
    #       UBool ignoreDstAmount,
    #       UErrorCode &ec
    # )
    # From icu/source/test/intltest/tzrulets.cpp:
    # TimeZoneRuleTest::TestHasEquivalentTransitions()
    zone5 = TimeZone.create_time_zone("America/New_York")
    zone6 = TimeZone.create_time_zone("America/Indiana/Indianapolis")
    start = 1104537600000.0  # 2005-01-01T00:00:00Z
    end = 1262304000000.0  # 2010-01-01T00:00:00Z
    assert not zone5.has_equivalent_transitions(zone6, start, end, True)
    start = 1136073600000.0  # 2006-01-01T00:00:00Z
    assert zone5.has_equivalent_transitions(zone6, start, end, True)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 69, reason="ICU4C<69")
def test_basic_time_zone_get_offset_from_local():
    from icupy import UTimeZoneLocalOption

    # From icu/source/test/intltest/tzoffloc.cpp:
    # TimeZoneOffsetLocalTest::TestGetOffsetAroundTransition()
    zone = TimeZone.create_time_zone("America/Los_Angeles")
    assert isinstance(zone, BasicTimeZone)

    # void icu::BasicTimeZone::getOffsetFromLocal(
    #       UDate date,
    #       UTimeZoneLocalOption nonExistingTimeOpt,
    #       UTimeZoneLocalOption duplicatedTimeOpt,
    #       int32_t &rawOffset,
    #       int32_t &dstOffset,
    #       UErrorCode &status
    # )
    date = 1143941400000  # 2006-04-02T01:30:00Z
    raw_offset, dst_offset = zone.get_offset_from_local(
        date,
        UTimeZoneLocalOption.UCAL_TZ_LOCAL_STANDARD_FORMER,
        UTimeZoneLocalOption.UCAL_TZ_LOCAL_STANDARD_LATTER)
    assert raw_offset == -8 * HOUR
    assert dst_offset == 0


def test_rule_based_time_zone():
    assert issubclass(RuleBasedTimeZone, BasicTimeZone)
    assert issubclass(RuleBasedTimeZone, TimeZone)
    initial_rule1 = InitialTimeZoneRule("STD", 0, 0)
    initial_rule2 = InitialTimeZoneRule("STD", 5 * HOUR, 0)

    # [1]
    # icu::RuleBasedTimeZone::RuleBasedTimeZone(
    #       const UnicodeString &id,
    #       InitialTimeZoneRule *initialRule
    # )
    zone1 = RuleBasedTimeZone(UnicodeString("r1"), initial_rule1)
    dr11 = DateTimeRule(
        UCalendarMonths.UCAL_JULY,
        1,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        0,
        DateTimeRule.WALL_TIME)
    tzr11 = AnnualTimeZoneRule(
        "r1(STD)",
        0,
        0,
        dr11,
        0,
        AnnualTimeZoneRule.MAX_YEAR)
    zone1.add_transition_rule(tzr11)
    dr12 = DateTimeRule(
        UCalendarMonths.UCAL_JANUARY,
        1,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        0,
        DateTimeRule.WALL_TIME)
    tzr12 = AnnualTimeZoneRule(
        "r1(DST)",
        0,
        1 * HOUR,
        dr12,
        0,
        AnnualTimeZoneRule.MAX_YEAR)
    zone1.add_transition_rule(tzr12)
    zone1.complete()

    zone2 = RuleBasedTimeZone(UnicodeString("r2"), initial_rule2)
    dr21 = DateTimeRule(
        UCalendarMonths.UCAL_OCTOBER,
        -1,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        5 * HOUR,
        DateTimeRule.WALL_TIME)
    tzr21 = AnnualTimeZoneRule(
        "r2(STD)",
        5 * HOUR,
        0,
        dr21,
        2000,
        2010)
    zone2.add_transition_rule(tzr21)
    dr22 = DateTimeRule(
        UCalendarMonths.UCAL_MARCH,
        -1,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        2 * HOUR,
        DateTimeRule.WALL_TIME)
    tzr22 = AnnualTimeZoneRule(
        "r2(DST)",
        5 * HOUR,
        1 * HOUR,
        dr22,
        2000,
        2010)
    zone2.add_transition_rule(tzr22)
    dr23 = DateTimeRule(
        UCalendarMonths.UCAL_JANUARY,
        1,
        0,
        DateTimeRule.WALL_TIME)
    tzr23 = AnnualTimeZoneRule(
        "r2(STD)",
        6 * HOUR,
        0,
        dr23,
        2011,
        AnnualTimeZoneRule.MAX_YEAR)
    zone2.add_transition_rule(tzr23)
    dr24 = DateTimeRule(
        UCalendarMonths.UCAL_JANUARY,
        1,
        1,
        DateTimeRule.WALL_TIME)
    tzr24 = AnnualTimeZoneRule(
        "r2(DST)",
        6 * HOUR,
        0,
        dr24,
        2011,
        AnnualTimeZoneRule.MAX_YEAR)
    zone2.add_transition_rule(tzr24)
    zone2.complete()

    # [2]
    # icu::RuleBasedTimeZone::RuleBasedTimeZone(
    #       const RuleBasedTimeZone &source
    # )
    zone3 = RuleBasedTimeZone(zone2)

    # RuleBasedTimeZone *icu::RuleBasedTimeZone::clone()
    zone2a = zone2.clone()
    assert isinstance(zone2a, RuleBasedTimeZone)
    assert zone2a == zone2

    zone2b = copy.copy(zone2)
    assert zone2b == zone2

    zone2c = copy.deepcopy(zone2)
    assert zone2c == zone2

    # UBool icu::TimeZone::operator!=(const TimeZone &that)
    assert zone1 != zone2
    assert zone1 != zone3
    assert not (zone2 != zone3)

    # UBool icu::RuleBasedTimeZone::operator==(const TimeZone &that)
    assert not (zone1 == zone2)
    assert not (zone1 == zone3)
    assert zone2 == zone3

    # int32_t icu::BasicTimeZone::countTransitionRules(UErrorCode &status)
    assert zone1.count_transition_rules() == 2
    assert zone2.count_transition_rules() == 4

    # int32_t icu::TimeZone::getDSTSavings()
    assert zone1.get_dst_savings() == 1 * HOUR
    assert zone2.get_dst_savings() == 0

    # UnicodeString &icu::TimeZone::getID(UnicodeString &ID)
    id_ = UnicodeString()
    result = zone1.get_id(id_)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(id_)
    assert result == "r1"

    # UBool icu::BasicTimeZone::getNextTransition(
    #       UDate base,
    #       UBool inclusive,
    #       TimeZoneTransition &result
    # )
    base = 1230681600000.0  # 2008-12-31T00:00:00Z
    tzt = TimeZoneTransition()
    assert zone1.get_next_transition(base, False, tzt)
    assert tzt.get_time() == 1231027200000.0  # 2009-01-04T00:00:00Z

    # UBool icu::BasicTimeZone::getPreviousTransition(
    #       UDate base,
    #       UBool inclusive,
    #       TimeZoneTransition &result
    # )
    assert zone1.get_previous_transition(base, False, tzt)
    assert tzt.get_time() == 1215298800000.0  # 2008-07-05T23:00:00Z

    # int32_t icu::TimeZone::getRawOffset(void)
    assert zone1.get_raw_offset() == 0
    assert zone2.get_raw_offset() == 6 * HOUR

    # void icu::BasicTimeZone::getSimpleRulesNear(
    #       UDate date,
    #       InitialTimeZoneRule *&initial,
    #       AnnualTimeZoneRule *&std,
    #       AnnualTimeZoneRule *&dst,
    #       UErrorCode &status
    # )
    initial, std, dst = zone1.get_simple_rules_near(0)
    assert isinstance(initial, InitialTimeZoneRule)
    assert initial.get_raw_offset() == 0
    assert initial.get_dst_savings() == 1 * HOUR
    assert isinstance(std, AnnualTimeZoneRule)
    assert std.get_raw_offset() == 0
    assert std.get_dst_savings() == 0
    dr = std.get_rule()
    assert dr.get_rule_month() == UCalendarMonths.UCAL_JULY
    assert dr.get_rule_day_of_month() == 0
    assert isinstance(dst, AnnualTimeZoneRule)
    assert dst.get_raw_offset() == 0
    assert dst.get_dst_savings() == 1 * HOUR
    dr = dst.get_rule()
    assert dr.get_rule_month() == UCalendarMonths.UCAL_JANUARY
    assert dr.get_rule_day_of_month() == 0

    # void icu::BasicTimeZone::getTimeZoneRules(
    #       const InitialTimeZoneRule *&initial,
    #       const TimeZoneRule *trsrules[],
    #       int32_t &trscount,
    #       UErrorCode &status
    # )
    initial, trsrules = zone2.get_time_zone_rules()
    assert isinstance(initial, InitialTimeZoneRule)
    assert initial.get_raw_offset() == 5 * HOUR
    assert initial.get_dst_savings() == 0
    assert isinstance(trsrules, list)
    assert len(trsrules) == 4
    assert all(isinstance(x, AnnualTimeZoneRule) for x in trsrules)
    assert trsrules[0] == tzr21
    assert trsrules[1] == tzr22
    assert trsrules[2] == tzr23
    assert trsrules[3] == tzr24

    # UBool icu::BasicTimeZone::hasEquivalentTransitions(
    #       const BasicTimeZone &tz,
    #       UDate start,
    #       UDate end,
    #       UBool ignoreDstAmount,
    #       UErrorCode &ec
    # )
    start = 1215298800000.0  # 2008-07-05T23:00:00Z
    end = 1231027200000.0  # 2009-01-04T00:00:00Z
    assert not zone1.has_equivalent_transitions(zone2, start, end, False)
    assert not zone2.has_equivalent_transitions(zone1, start, end, False)

    # UBool icu::TimeZone::hasSameRules(const TimeZone &other)
    assert not zone1.has_same_rules(zone2)
    assert zone2.has_same_rules(zone3)

    # void icu::TimeZone::setID(const UnicodeString &ID)
    zone1.set_id(UnicodeString("abc"))
    assert zone1.get_id(id_) == "abc"
    zone1.set_id("ABC")
    assert zone1.get_id(id_) == "ABC"

    # void icu::TimeZone::setRawOffset(int32_t offsetMillis)
    zone1.set_raw_offset(-5 * HOUR)  # Nothing to do
    assert zone1.get_raw_offset() == 0

    # UBool icu::TimeZone::useDaylightTime(void)
    assert zone1.use_daylight_time()
    assert not zone2.use_daylight_time()


def test_rule_based_time_zone_get_offset():
    initial_rule = InitialTimeZoneRule(
        "r1(STD)",
        8 * HOUR,
        0)
    zone = RuleBasedTimeZone("r1", initial_rule)
    dr1 = DateTimeRule(
        UCalendarMonths.UCAL_OCTOBER,
        -1,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        2 * HOUR,
        DateTimeRule.WALL_TIME)
    tzr1 = AnnualTimeZoneRule(
        "r1(STD)",
        8 * HOUR,
        0,
        dr1,
        0,
        AnnualTimeZoneRule.MAX_YEAR)
    zone.add_transition_rule(tzr1)
    dr2 = DateTimeRule(
        UCalendarMonths.UCAL_APRIL,
        1,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        True,
        2 * HOUR,
        DateTimeRule.WALL_TIME)
    tzr2 = AnnualTimeZoneRule(
        "r1(DST)",
        8 * HOUR,
        int(0.5 * HOUR),
        dr2,
        0,
        AnnualTimeZoneRule.MAX_YEAR)
    zone.add_transition_rule(tzr2)
    zone.complete()

    # [1]
    # void icu::RuleBasedTimeZone::getOffset(
    #       UDate date,
    #       UBool local,
    #       int32_t &rawOffset,
    #       int32_t &dstOffset,
    #       UErrorCode &ec
    # )
    raw_offset, dst_offset = zone.get_offset(0, False)
    assert raw_offset == 8 * HOUR
    assert dst_offset == 0

    # [2]
    # int32_t icu::RuleBasedTimeZone::getOffset(
    #       uint8_t era,
    #       int32_t year,
    #       int32_t month,
    #       int32_t day,
    #       uint8_t dayOfWeek,
    #       int32_t millis,
    #       int32_t monthLength,
    #       UErrorCode &status
    # )
    assert zone.get_offset(
        GregorianCalendar.AD,
        2008,
        UCalendarMonths.UCAL_JULY,
        6,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        0,
        28) == 8.5 * HOUR

    # [3]
    # int32_t icu::RuleBasedTimeZone::getOffset(
    #       uint8_t era,
    #       int32_t year,
    #       int32_t month,
    #       int32_t day,
    #       uint8_t dayOfWeek,
    #       int32_t millis,
    #       UErrorCode &status
    # )
    assert zone.get_offset(
        GregorianCalendar.AD,
        2008,
        UCalendarMonths.UCAL_JULY,
        6,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        0) == 8.5 * HOUR


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 69, reason="ICU4C<69")
def test_rule_based_time_zone_get_offset_from_local():
    from icupy import UTimeZoneLocalOption

    # From icu/source/test/intltest/tzoffloc.cpp:
    # TimeZoneOffsetLocalTest::TestGetOffsetAroundTransition()
    zone = RuleBasedTimeZone(
        "Rule based Pacific Time",
        InitialTimeZoneRule("Pacific Standard Time", -8 * HOUR, 0)
    )
    zone.add_transition_rule(
        AnnualTimeZoneRule(
            "Pacific Daylight Time",
            -8 * HOUR,
            1 * HOUR,
            DateTimeRule(
                UCalendarMonths.UCAL_APRIL,
                1,
                UCalendarDaysOfWeek.UCAL_SUNDAY,
                2 * HOUR,
                DateTimeRule.WALL_TIME
            ),
            2000,
            AnnualTimeZoneRule.MAX_YEAR
        )
    )
    zone.add_transition_rule(
        AnnualTimeZoneRule(
            "Pacific Standard Time",
            -8 * HOUR,
            0,
            DateTimeRule(
                UCalendarMonths.UCAL_OCTOBER,
                -1,
                UCalendarDaysOfWeek.UCAL_SUNDAY,
                2 * HOUR,
                DateTimeRule.WALL_TIME
            ),
            2000,
            AnnualTimeZoneRule.MAX_YEAR
        )
    )
    zone.complete()

    # void icu::RuleBasedTimeZone::getOffsetFromLocal(
    #       UDate date,
    #       UTimeZoneLocalOption nonExistingTimeOpt,
    #       UTimeZoneLocalOption duplicatedTimeOpt,
    #       int32_t &rawOffset,
    #       int32_t &dstOffset,
    #       UErrorCode &status
    # )
    date = 1143941400000  # 2006-04-02T01:30:00Z
    raw_offset, dst_offset = zone.get_offset_from_local(
        date,
        UTimeZoneLocalOption.UCAL_TZ_LOCAL_STANDARD_FORMER,
        UTimeZoneLocalOption.UCAL_TZ_LOCAL_STANDARD_LATTER)
    assert raw_offset == -8 * HOUR
    assert dst_offset == 0


def test_simple_time_zone():
    assert issubclass(SimpleTimeZone, BasicTimeZone)
    assert issubclass(SimpleTimeZone, TimeZone)

    # [2]
    # icu::SimpleTimeZone::SimpleTimeZone(
    #       int32_t rawOffsetGMT,
    #       const UnicodeString &ID
    # )
    zone2 = SimpleTimeZone(8 * HOUR, UnicodeString("s2"))

    # [3]
    # icu::SimpleTimeZone::SimpleTimeZone(
    #       int32_t rawOffsetGMT,
    #       const UnicodeString &ID,
    #       int8_t savingsStartMonth,
    #       int8_t savingsStartDayOfWeekInMonth,
    #       int8_t savingsStartDayOfWeek,
    #       int32_t savingsStartTime,
    #       int8_t savingsEndMonth,
    #       int8_t savingsEndDayOfWeekInMonth,
    #       int8_t savingsEndDayOfWeek,
    #       int32_t savingsEndTime,
    #       UErrorCode &status
    # )
    zone3 = SimpleTimeZone(
        0,
        UnicodeString("s3"),
        UCalendarMonths.UCAL_JANUARY,
        1,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        0,
        UCalendarMonths.UCAL_JULY,
        1,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        0)

    # [4]
    # icu::SimpleTimeZone::SimpleTimeZone(
    #       int32_t rawOffsetGMT,
    #       const UnicodeString &ID,
    #       int8_t savingsStartMonth,
    #       int8_t savingsStartDayOfWeekInMonth,
    #       int8_t savingsStartDayOfWeek,
    #       int32_t savingsStartTime,
    #       int8_t savingsEndMonth,
    #       int8_t savingsEndDayOfWeekInMonth,
    #       int8_t savingsEndDayOfWeek,
    #       int32_t savingsEndTime,
    #       int32_t savingsDST,
    #       UErrorCode &status
    # )
    zone4 = SimpleTimeZone(
        -5 * HOUR,
        UnicodeString("s4"),
        UCalendarMonths.UCAL_MARCH,
        1,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        0,
        UCalendarMonths.UCAL_SEPTEMBER,
        1,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        0,
        int(0.5 * HOUR))

    # [5]
    # icu::SimpleTimeZone::SimpleTimeZone(
    #       int32_t rawOffsetGMT,
    #       const UnicodeString &ID,
    #       int8_t savingsStartMonth,
    #       int8_t savingsStartDayOfWeekInMonth,
    #       int8_t savingsStartDayOfWeek,
    #       int32_t savingsStartTime,
    #       TimeMode savingsStartTimeMode,
    #       int8_t savingsEndMonth,
    #       int8_t savingsEndDayOfWeekInMonth,
    #       int8_t savingsEndDayOfWeek,
    #       int32_t savingsEndTime,
    #       TimeMode savingsEndTimeMode,
    #       int32_t savingsDST,
    #       UErrorCode &status
    # )
    zone5 = SimpleTimeZone(
        8 * HOUR,
        UnicodeString("s5"),
        UCalendarMonths.UCAL_APRIL,
        1,
        -1,
        2 * HOUR,
        SimpleTimeZone.WALL_TIME,
        UCalendarMonths.UCAL_OCTOBER,
        -1,
        1,
        2 * HOUR,
        SimpleTimeZone.WALL_TIME,
        int(0.5 * HOUR))

    # [1]
    # icu::SimpleTimeZone::SimpleTimeZone(const SimpleTimeZone &source)
    zone1 = SimpleTimeZone(zone5)

    # SimpleTimeZone *icu::SimpleTimeZone::clone()
    zone1a = zone1.clone()
    assert isinstance(zone1a, SimpleTimeZone)
    assert zone1a == zone1

    zone1b = copy.copy(zone1)
    assert zone1b == zone1

    zone1c = copy.deepcopy(zone1)
    assert zone1c == zone1

    # UBool icu::TimeZone::operator!=(const TimeZone &that)
    assert zone1 != zone2
    assert zone1 != zone3
    assert zone1 != zone4
    assert not (zone1 != zone5)
    assert zone2 != zone3
    assert zone2 != zone4
    assert zone2 != zone5
    assert zone3 != zone4
    assert zone3 != zone5
    assert zone4 != zone5

    # UBool icu::SimpleTimeZone::operator==(const TimeZone &that)
    assert not (zone1 == zone2)
    assert not (zone1 == zone3)
    assert not (zone1 == zone4)
    assert zone1 == zone5
    assert not (zone2 == zone3)
    assert not (zone2 == zone4)
    assert not (zone2 == zone5)
    assert not (zone3 == zone4)
    assert not (zone3 == zone5)
    assert not (zone4 == zone5)

    # int32_t icu::BasicTimeZone::countTransitionRules(UErrorCode &status)
    assert zone1.count_transition_rules() == 2
    assert zone2.count_transition_rules() == 0
    assert zone3.count_transition_rules() == 2
    assert zone4.count_transition_rules() == 2
    assert zone5.count_transition_rules() == 2

    # int32_t icu::TimeZone::getDSTSavings()
    assert zone1.get_dst_savings() == 0.5 * HOUR
    assert zone2.get_dst_savings() == 1 * HOUR
    assert zone3.get_dst_savings() == 1 * HOUR
    assert zone4.get_dst_savings() == 0.5 * HOUR
    assert zone5.get_dst_savings() == 0.5 * HOUR

    # UnicodeString &icu::TimeZone::getID(UnicodeString &ID)
    id_ = UnicodeString()
    result = zone1.get_id(id_)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(id_)
    assert result == "s5"

    # UBool icu::BasicTimeZone::getNextTransition(
    #       UDate base,
    #       UBool inclusive,
    #       TimeZoneTransition &result
    # )
    base = 1230681600000.0  # 2008-12-31T00:00:00Z
    tzt = TimeZoneTransition()
    assert zone3.get_next_transition(base, False, tzt)
    assert tzt.get_time() == 1231027200000.0  # 2009-01-04T00:00:00Z

    # UBool icu::BasicTimeZone::getPreviousTransition(
    #       UDate base,
    #       UBool inclusive,
    #       TimeZoneTransition &result
    # )
    assert zone3.get_previous_transition(base, False, tzt)
    assert tzt.get_time() == 1215298800000.0  # 2008-07-05T23:00:00Z

    # int32_t icu::TimeZone::getRawOffset(void)
    assert zone1.get_raw_offset() == 8 * HOUR
    assert zone2.get_raw_offset() == 8 * HOUR
    assert zone3.get_raw_offset() == 0
    assert zone4.get_raw_offset() == -5 * HOUR
    assert zone5.get_raw_offset() == 8 * HOUR

    # void icu::BasicTimeZone::getSimpleRulesNear(
    #       UDate date,
    #       InitialTimeZoneRule *&initial,
    #       AnnualTimeZoneRule *&std,
    #       AnnualTimeZoneRule *&dst,
    #       UErrorCode &status
    # )
    initial, std, dst = zone2.get_simple_rules_near(0)
    assert isinstance(initial, InitialTimeZoneRule)
    assert initial.get_raw_offset() == 8 * HOUR
    assert initial.get_dst_savings() == 0
    assert std is None
    assert dst is None

    initial, std, dst = zone4.get_simple_rules_near(0)
    assert isinstance(initial, InitialTimeZoneRule)
    assert initial.get_raw_offset() == -5 * HOUR
    assert initial.get_dst_savings() == 0.5 * HOUR
    assert isinstance(std, AnnualTimeZoneRule)
    assert std.get_raw_offset() == -5 * HOUR
    assert std.get_dst_savings() == 0
    dr = std.get_rule()
    assert dr.get_rule_month() == UCalendarMonths.UCAL_SEPTEMBER
    assert dr.get_rule_day_of_month() == 0
    assert isinstance(dst, AnnualTimeZoneRule)
    assert dst.get_raw_offset() == -5 * HOUR
    assert dst.get_dst_savings() == 0.5 * HOUR
    dr = dst.get_rule()
    assert dr.get_rule_month() == UCalendarMonths.UCAL_MARCH
    assert dr.get_rule_day_of_month() == 0

    # void icu::BasicTimeZone::getTimeZoneRules(
    #       const InitialTimeZoneRule *&initial,
    #       const TimeZoneRule *trsrules[],
    #       int32_t &trscount,
    #       UErrorCode &status
    # )
    initial, trsrules = zone2.get_time_zone_rules()
    assert isinstance(initial, InitialTimeZoneRule)
    assert initial.get_raw_offset() == 8 * HOUR
    assert initial.get_dst_savings() == 0
    assert isinstance(trsrules, list)
    assert len(trsrules) == 0

    initial, trsrules = zone4.get_time_zone_rules()
    assert isinstance(initial, InitialTimeZoneRule)
    assert initial.get_raw_offset() == -5 * HOUR
    assert initial.get_dst_savings() == 0
    assert isinstance(trsrules, list)
    assert len(trsrules) == 2
    assert all(isinstance(x, AnnualTimeZoneRule) for x in trsrules)
    assert trsrules[0].get_raw_offset() == -5 * HOUR
    assert trsrules[0].get_dst_savings() == 0
    dr = trsrules[0].get_rule()
    assert dr.get_rule_month() == UCalendarMonths.UCAL_SEPTEMBER
    assert dr.get_rule_day_of_month() == 0
    assert trsrules[1].get_raw_offset() == -5 * HOUR
    assert trsrules[1].get_dst_savings() == 0.5 * HOUR
    dr = trsrules[1].get_rule()
    assert dr.get_rule_month() == UCalendarMonths.UCAL_MARCH
    assert dr.get_rule_day_of_month() == 0

    # UBool icu::BasicTimeZone::hasEquivalentTransitions(
    #       const BasicTimeZone &tz,
    #       UDate start,
    #       UDate end,
    #       UBool ignoreDstAmount,
    #       UErrorCode &ec
    # )
    start = 1215298800000.0  # 2008-07-05T23:00:00Z
    end = 1231027200000.0  # 2009-01-04T00:00:00Z
    assert not zone5.has_equivalent_transitions(zone4, start, end, False)
    assert zone5.has_equivalent_transitions(zone1, start, end, False)

    # UBool icu::TimeZone::hasSameRules(const TimeZone &other)
    assert not zone5.has_same_rules(zone4)
    assert zone5.has_same_rules(zone1)

    # void icu::SimpleTimeZone::setDSTSavings(
    #       int32_t millisSavedDuringDST,
    #       UErrorCode &status
    # )
    zone1.set_dst_savings(1 * HOUR)
    assert zone1.get_dst_savings() == 1 * HOUR
    zone2.set_dst_savings(int(0.5 * HOUR))
    assert zone2.get_dst_savings() == 0.5 * HOUR

    # void icu::TimeZone::setID(const UnicodeString &ID)
    zone1.set_id(UnicodeString("abc"))
    assert zone1.get_id(id_) == "abc"
    zone1.set_id("ABC")
    assert zone1.get_id(id_) == "ABC"

    # void icu::TimeZone::setRawOffset(int32_t offsetMillis)
    zone1.set_raw_offset(0)
    assert zone1.get_raw_offset() == 0
    zone2.set_raw_offset(-5 * HOUR)
    assert zone2.get_raw_offset() == -5 * HOUR

    _, trsrules = zone1.get_time_zone_rules()
    assert isinstance(trsrules, list)
    assert len(trsrules) == 2
    assert trsrules[0].get_start_year() == 0
    assert trsrules[1].get_start_year() == 0
    zone1.set_start_year(1966)
    _, trsrules = zone1.get_time_zone_rules()
    assert isinstance(trsrules, list)
    assert len(trsrules) == 2
    assert trsrules[0].get_start_year() == 1966
    assert trsrules[1].get_start_year() == 1966

    # UBool icu::TimeZone::useDaylightTime(void)
    assert zone1.use_daylight_time()
    assert not zone2.use_daylight_time()
    assert zone3.use_daylight_time()
    assert zone4.use_daylight_time()
    assert zone5.use_daylight_time()


def test_simple_time_zone_get_offset():
    zone = SimpleTimeZone(
        8 * HOUR,
        "s5",
        UCalendarMonths.UCAL_APRIL,
        1,
        -1,
        2 * HOUR,
        SimpleTimeZone.WALL_TIME,
        UCalendarMonths.UCAL_OCTOBER,
        -1,
        1,
        2 * HOUR,
        SimpleTimeZone.WALL_TIME,
        int(0.5 * HOUR))

    # [1]
    # void icu::SimpleTimeZone::getOffset(
    #       UDate date,
    #       UBool local,
    #       int32_t &rawOffset,
    #       int32_t &dstOffset,
    #       UErrorCode &ec
    # )
    raw_offset, dst_offset = zone.get_offset(0, False)
    assert raw_offset == 8 * HOUR
    assert dst_offset == 0

    # [2]
    # int32_t icu::SimpleTimeZone::getOffset(
    #       uint8_t era,
    #       int32_t year,
    #       int32_t month,
    #       int32_t day,
    #       uint8_t dayOfWeek,
    #       int32_t millis,
    #       UErrorCode &status
    # )
    assert zone.get_offset(
        GregorianCalendar.AD,
        2008,
        UCalendarMonths.UCAL_JULY,
        6,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        0) == 8.5 * HOUR

    # [3]
    # int32_t icu::SimpleTimeZone::getOffset(
    #       uint8_t era,
    #       int32_t year,
    #       int32_t month,
    #       int32_t day,
    #       uint8_t dayOfWeek,
    #       int32_t milliseconds,
    #       int32_t monthLength,
    #       int32_t prevMonthLength,
    #       UErrorCode &status
    # )
    assert zone.get_offset(
        GregorianCalendar.AD,
        2008,
        UCalendarMonths.UCAL_JULY,
        6,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        0,
        28,
        28) == 8.5 * HOUR

    # [4]
    # int32_t icu::SimpleTimeZone::getOffset(
    #       uint8_t era,
    #       int32_t year,
    #       int32_t month,
    #       int32_t day,
    #       uint8_t dayOfWeek,
    #       int32_t milliseconds,
    #       int32_t monthLength,
    #       UErrorCode &status
    # )
    assert zone.get_offset(
        GregorianCalendar.AD,
        2008,
        UCalendarMonths.UCAL_JULY,
        6,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        0,
        28) == 8.5 * HOUR


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 69, reason="ICU4C<69")
def test_simple_time_zone_get_offset_from_local():
    from icupy import UTimeZoneLocalOption

    # From icu/source/test/intltest/tzoffloc.cpp:
    # TimeZoneOffsetLocalTest::TestGetOffsetAroundTransition()
    zone = SimpleTimeZone(
        -8 * HOUR,
        "Simple Pacific Time",
        UCalendarMonths.UCAL_APRIL,
        1,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        2 * HOUR,
        UCalendarMonths.UCAL_OCTOBER,
        -1,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        2 * HOUR)

    # void icu::SimpleTimeZone::getOffsetFromLocal(
    #       UDate date,
    #       UTimeZoneLocalOption nonExistingTimeOpt,
    #       UTimeZoneLocalOption duplicatedTimeOpt,
    #       int32_t &rawOffset,
    #       int32_t &dstOffset,
    #       UErrorCode &status
    # )
    date = 1143941400000  # 2006-04-02T01:30:00Z
    raw_offset, dst_offset = zone.get_offset_from_local(
        date,
        UTimeZoneLocalOption.UCAL_TZ_LOCAL_STANDARD_FORMER,
        UTimeZoneLocalOption.UCAL_TZ_LOCAL_STANDARD_LATTER)
    assert raw_offset == -8 * HOUR
    assert dst_offset == 0


def test_simple_time_zone_set_end_rule():
    zone = SimpleTimeZone(
        8 * HOUR,
        "s5",
        UCalendarMonths.UCAL_APRIL,
        1,
        -1,
        2 * HOUR,
        SimpleTimeZone.WALL_TIME,
        UCalendarMonths.UCAL_OCTOBER,
        -1,
        1,
        2 * HOUR,
        SimpleTimeZone.WALL_TIME,
        int(0.5 * HOUR))

    # [1]
    # void icu::SimpleTimeZone::setEndRule(
    #       int32_t month,
    #       int32_t dayOfMonth,
    #       int32_t dayOfWeek,
    #       int32_t time,
    #       TimeMode mode,
    #       UBool after,
    #       UErrorCode &status
    # )
    zone.set_end_rule(
        UCalendarMonths.UCAL_DECEMBER,
        31,
        UCalendarDaysOfWeek.UCAL_SATURDAY,
        6 * HOUR,
        SimpleTimeZone.UTC_TIME,
        True)
    _, trsrules = zone.get_time_zone_rules()
    assert isinstance(trsrules, list)
    assert len(trsrules) == 2
    dr = trsrules[0].get_rule()
    assert dr.get_date_rule_type() == DateTimeRule.DOW_GEQ_DOM
    assert dr.get_rule_day_of_month() == 31
    assert dr.get_rule_day_of_week() == UCalendarDaysOfWeek.UCAL_SATURDAY
    assert dr.get_rule_millis_in_day() == 6 * HOUR
    assert dr.get_rule_month() == UCalendarMonths.UCAL_DECEMBER
    assert dr.get_rule_week_in_month() == 0
    assert dr.get_time_rule_type() == DateTimeRule.UTC_TIME

    # [2]
    # void icu::SimpleTimeZone::setEndRule(
    #       int32_t month,
    #       int32_t dayOfMonth,
    #       int32_t dayOfWeek,
    #       int32_t time,
    #       UBool after,
    #       UErrorCode &status
    # )
    zone.set_end_rule(
        UCalendarMonths.UCAL_OCTOBER,
        31,
        UCalendarDaysOfWeek.UCAL_FRIDAY,
        5 * HOUR,
        True)
    _, trsrules = zone.get_time_zone_rules()
    assert isinstance(trsrules, list)
    assert len(trsrules) == 2
    dr = trsrules[0].get_rule()
    assert dr.get_date_rule_type() == DateTimeRule.DOW_GEQ_DOM
    assert dr.get_rule_day_of_month() == 31
    assert dr.get_rule_day_of_week() == UCalendarDaysOfWeek.UCAL_FRIDAY
    assert dr.get_rule_millis_in_day() == 5 * HOUR
    assert dr.get_rule_month() == UCalendarMonths.UCAL_OCTOBER
    assert dr.get_rule_week_in_month() == 0
    assert dr.get_time_rule_type() == DateTimeRule.WALL_TIME

    # [3]
    # void icu::SimpleTimeZone::setEndRule(
    #       int32_t month,
    #       int32_t dayOfMonth,
    #       int32_t time,
    #       TimeMode mode,
    #       UErrorCode &status
    # )
    zone.set_end_rule(
        UCalendarMonths.UCAL_SEPTEMBER,
        30,
        4 * HOUR,
        SimpleTimeZone.UTC_TIME)
    _, trsrules = zone.get_time_zone_rules()
    assert isinstance(trsrules, list)
    assert len(trsrules) == 2
    dr = trsrules[0].get_rule()
    assert dr.get_date_rule_type() == DateTimeRule.DOM
    assert dr.get_rule_day_of_month() == 30
    assert dr.get_rule_day_of_week() == 0
    assert dr.get_rule_millis_in_day() == 4 * HOUR
    assert dr.get_rule_month() == UCalendarMonths.UCAL_SEPTEMBER
    assert dr.get_rule_week_in_month() == 0
    assert dr.get_time_rule_type() == DateTimeRule.UTC_TIME

    # [4]
    # void icu::SimpleTimeZone::setEndRule(
    #       int32_t month,
    #       int32_t dayOfMonth,
    #       int32_t time,
    #       UErrorCode &status
    # )
    zone.set_end_rule(
        UCalendarMonths.UCAL_AUGUST,
        31,
        3 * HOUR)
    _, trsrules = zone.get_time_zone_rules()
    assert isinstance(trsrules, list)
    assert len(trsrules) == 2
    dr = trsrules[0].get_rule()
    assert dr.get_date_rule_type() == DateTimeRule.DOM
    assert dr.get_rule_day_of_month() == 31
    assert dr.get_rule_day_of_week() == 0
    assert dr.get_rule_millis_in_day() == 3 * HOUR
    assert dr.get_rule_month() == UCalendarMonths.UCAL_AUGUST
    assert dr.get_rule_week_in_month() == 0
    assert dr.get_time_rule_type() == DateTimeRule.WALL_TIME

    # [5]
    # void icu::SimpleTimeZone::setEndRule(
    #       int32_t month,
    #       int32_t dayOfWeekInMonth,
    #       int32_t dayOfWeek,
    #       int32_t time,
    #       TimeMode mode,
    #       UErrorCode &status
    # )
    zone.set_end_rule(
        UCalendarMonths.UCAL_JULY,
        -1,
        UCalendarDaysOfWeek.UCAL_THURSDAY,
        2 * HOUR,
        SimpleTimeZone.UTC_TIME)
    _, trsrules = zone.get_time_zone_rules()
    assert isinstance(trsrules, list)
    assert len(trsrules) == 2
    dr = trsrules[0].get_rule()
    assert dr.get_date_rule_type() == DateTimeRule.DOW
    assert dr.get_rule_day_of_month() == 0
    assert dr.get_rule_day_of_week() == UCalendarDaysOfWeek.UCAL_THURSDAY
    assert dr.get_rule_millis_in_day() == 2 * HOUR
    assert dr.get_rule_month() == UCalendarMonths.UCAL_JULY
    assert dr.get_rule_week_in_month() == -1
    assert dr.get_time_rule_type() == DateTimeRule.UTC_TIME

    # [6]
    # void icu::SimpleTimeZone::setEndRule(
    #       int32_t month,
    #       int32_t dayOfWeekInMonth,
    #       int32_t dayOfWeek,
    #       int32_t time,
    #       UErrorCode &status
    # )
    zone.set_end_rule(
        UCalendarMonths.UCAL_JUNE,
        2,
        UCalendarDaysOfWeek.UCAL_WEDNESDAY,
        1 * HOUR)
    _, trsrules = zone.get_time_zone_rules()
    assert isinstance(trsrules, list)
    assert len(trsrules) == 2
    dr = trsrules[0].get_rule()
    assert dr.get_date_rule_type() == DateTimeRule.DOW
    assert dr.get_rule_day_of_month() == 0
    assert dr.get_rule_day_of_week() == UCalendarDaysOfWeek.UCAL_WEDNESDAY
    assert dr.get_rule_millis_in_day() == 1 * HOUR
    assert dr.get_rule_month() == UCalendarMonths.UCAL_JUNE
    assert dr.get_rule_week_in_month() == 2
    assert dr.get_time_rule_type() == DateTimeRule.WALL_TIME


def test_simple_time_zone_set_start_rule():
    zone = SimpleTimeZone(
        8 * HOUR,
        "s5",
        UCalendarMonths.UCAL_APRIL,
        1,
        -1,
        2 * HOUR,
        SimpleTimeZone.WALL_TIME,
        UCalendarMonths.UCAL_OCTOBER,
        -1,
        1,
        2 * HOUR,
        SimpleTimeZone.WALL_TIME,
        int(0.5 * HOUR))

    # [1]
    # void icu::SimpleTimeZone::setStartRule(
    #       int32_t month,
    #       int32_t dayOfMonth,
    #       int32_t dayOfWeek,
    #       int32_t time,
    #       TimeMode mode,
    #       UBool after,
    #       UErrorCode &status
    # )
    zone.set_start_rule(
        UCalendarMonths.UCAL_JANUARY,
        31,
        UCalendarDaysOfWeek.UCAL_SATURDAY,
        1 * HOUR,
        SimpleTimeZone.UTC_TIME,
        True)
    _, trsrules = zone.get_time_zone_rules()
    assert isinstance(trsrules, list)
    assert len(trsrules) == 2
    dr = trsrules[1].get_rule()
    assert dr.get_date_rule_type() == DateTimeRule.DOW_GEQ_DOM
    assert dr.get_rule_day_of_month() == 31
    assert dr.get_rule_day_of_week() == UCalendarDaysOfWeek.UCAL_SATURDAY
    assert dr.get_rule_millis_in_day() == 1 * HOUR
    assert dr.get_rule_month() == UCalendarMonths.UCAL_JANUARY
    assert dr.get_rule_week_in_month() == 0
    assert dr.get_time_rule_type() == DateTimeRule.UTC_TIME

    # [2]
    # void icu::SimpleTimeZone::setStartRule(
    #       int32_t month,
    #       int32_t dayOfMonth,
    #       int32_t dayOfWeek,
    #       int32_t time,
    #       UBool after,
    #       UErrorCode &status
    # )
    zone.set_start_rule(
        UCalendarMonths.UCAL_FEBRUARY,
        28,
        UCalendarDaysOfWeek.UCAL_FRIDAY,
        2 * HOUR,
        True)
    _, trsrules = zone.get_time_zone_rules()
    assert isinstance(trsrules, list)
    assert len(trsrules) == 2
    dr = trsrules[1].get_rule()
    assert dr.get_date_rule_type() == DateTimeRule.DOW_GEQ_DOM
    assert dr.get_rule_day_of_month() == 28
    assert dr.get_rule_day_of_week() == UCalendarDaysOfWeek.UCAL_FRIDAY
    assert dr.get_rule_millis_in_day() == 2 * HOUR
    assert dr.get_rule_month() == UCalendarMonths.UCAL_FEBRUARY
    assert dr.get_rule_week_in_month() == 0
    assert dr.get_time_rule_type() == DateTimeRule.WALL_TIME

    # [3]
    # void icu::SimpleTimeZone::setStartRule(
    #       int32_t month,
    #       int32_t dayOfMonth,
    #       int32_t time,
    #       TimeMode mode,
    #       UErrorCode &status
    # )
    zone.set_start_rule(
        UCalendarMonths.UCAL_MARCH,
        31,
        3 * HOUR,
        SimpleTimeZone.UTC_TIME)
    _, trsrules = zone.get_time_zone_rules()
    assert isinstance(trsrules, list)
    assert len(trsrules) == 2
    dr = trsrules[1].get_rule()
    assert dr.get_date_rule_type() == DateTimeRule.DOM
    assert dr.get_rule_day_of_month() == 31
    assert dr.get_rule_day_of_week() == 0
    assert dr.get_rule_millis_in_day() == 3 * HOUR
    assert dr.get_rule_month() == UCalendarMonths.UCAL_MARCH
    assert dr.get_rule_week_in_month() == 0
    assert dr.get_time_rule_type() == DateTimeRule.UTC_TIME

    # [4]
    # void icu::SimpleTimeZone::setStartRule(
    #       int32_t month,
    #       int32_t dayOfMonth,
    #       int32_t time,
    #       UErrorCode &status
    # )
    zone.set_start_rule(
        UCalendarMonths.UCAL_APRIL,
        30,
        4 * HOUR)
    _, trsrules = zone.get_time_zone_rules()
    assert isinstance(trsrules, list)
    assert len(trsrules) == 2
    dr = trsrules[1].get_rule()
    assert dr.get_date_rule_type() == DateTimeRule.DOM
    assert dr.get_rule_day_of_month() == 30
    assert dr.get_rule_day_of_week() == 0
    assert dr.get_rule_millis_in_day() == 4 * HOUR
    assert dr.get_rule_month() == UCalendarMonths.UCAL_APRIL
    assert dr.get_rule_week_in_month() == 0
    assert dr.get_time_rule_type() == DateTimeRule.WALL_TIME

    # [5]
    # void icu::SimpleTimeZone::setStartRule(
    #       int32_t month,
    #       int32_t dayOfWeekInMonth,
    #       int32_t dayOfWeek,
    #       int32_t time,
    #       TimeMode mode,
    #       UErrorCode &status
    # )
    zone.set_start_rule(
        UCalendarMonths.UCAL_MAY,
        -1,
        UCalendarDaysOfWeek.UCAL_THURSDAY,
        5 * HOUR,
        SimpleTimeZone.UTC_TIME)
    _, trsrules = zone.get_time_zone_rules()
    assert isinstance(trsrules, list)
    assert len(trsrules) == 2
    dr = trsrules[1].get_rule()
    assert dr.get_date_rule_type() == DateTimeRule.DOW
    assert dr.get_rule_day_of_month() == 0
    assert dr.get_rule_day_of_week() == UCalendarDaysOfWeek.UCAL_THURSDAY
    assert dr.get_rule_millis_in_day() == 5 * HOUR
    assert dr.get_rule_month() == UCalendarMonths.UCAL_MAY
    assert dr.get_rule_week_in_month() == -1
    assert dr.get_time_rule_type() == DateTimeRule.UTC_TIME

    # [6]
    # void icu::SimpleTimeZone::setStartRule(
    #       int32_t month,
    #       int32_t dayOfWeekInMonth,
    #       int32_t dayOfWeek,
    #       int32_t time,
    #       UErrorCode &status
    # )
    zone.set_start_rule(
        UCalendarMonths.UCAL_JUNE,
        2,
        UCalendarDaysOfWeek.UCAL_WEDNESDAY,
        6 * HOUR)
    _, trsrules = zone.get_time_zone_rules()
    assert isinstance(trsrules, list)
    assert len(trsrules) == 2
    dr = trsrules[1].get_rule()
    assert dr.get_date_rule_type() == DateTimeRule.DOW
    assert dr.get_rule_day_of_month() == 0
    assert dr.get_rule_day_of_week() == UCalendarDaysOfWeek.UCAL_WEDNESDAY
    assert dr.get_rule_millis_in_day() == 6 * HOUR
    assert dr.get_rule_month() == UCalendarMonths.UCAL_JUNE
    assert dr.get_rule_week_in_month() == 2
    assert dr.get_time_rule_type() == DateTimeRule.WALL_TIME


def test_time_zone():
    id_ = UnicodeString("PST")

    # static TimeZone *icu::TimeZone::createDefault(void)
    zone1 = TimeZone.create_default()
    assert isinstance(zone1, BasicTimeZone)

    # static TimeZone *icu::TimeZone::createTimeZone(const UnicodeString &ID)
    zone2 = TimeZone.create_time_zone(id_)
    assert isinstance(zone2, BasicTimeZone)

    zone3 = TimeZone.create_time_zone("PST")
    assert isinstance(zone3, BasicTimeZone)
    assert not (zone2 != zone3)
    assert zone2 == zone3

    # static const TimeZone *icu::TimeZone::getGMT(void)
    zone4 = TimeZone.get_gmt()
    assert isinstance(zone4, BasicTimeZone)

    # static void icu::TimeZone::setDefault(const TimeZone &zone)
    TimeZone.set_default(zone1)

    # static int32_t icu::TimeZone::countEquivalentIDs(const UnicodeString &id)
    n = TimeZone.count_equivalent_ids(id_)
    assert n > 0
    ids = [TimeZone.get_equivalent_id(id_, i) for i in range(n)]
    assert all(isinstance(x, UnicodeString) for x in ids)
    assert "America/Los_Angeles" in ids

    assert TimeZone.count_equivalent_ids("PST") == n
    id2 = TimeZone.get_equivalent_id("PST", 0)
    assert ids[0] == id2

    # [1]
    # static StringEnumeration *icu::TimeZone::createEnumeration()
    it1 = TimeZone.create_enumeration()
    assert isinstance(it1, StringEnumeration)
    assert len(it1) > 0

    # [2]
    # static StringEnumeration *icu::TimeZone::createEnumeration(
    #       const char *country
    # )
    it2 = TimeZone.create_enumeration(None)
    assert isinstance(it2, StringEnumeration)
    assert len(it2) > 0

    it3 = TimeZone.create_enumeration("US")
    assert isinstance(it3, StringEnumeration)
    assert len(it3) > 0

    # [3]
    # static StringEnumeration *icu::TimeZone::createEnumeration(
    #       int32_t rawOffset
    # )
    it4 = TimeZone.create_enumeration(-8 * HOUR)
    assert isinstance(it4, StringEnumeration)
    assert len(it4) > 0

    assert len(it1) == len(it2)
    assert len(it1) > len(it3) > len(it4)
    assert "America/Los_Angeles" in it3
    assert "America/Los_Angeles" in it4

    # static StringEnumeration *icu::TimeZone::createTimeZoneIDEnumeration(
    #       USystemTimeZoneType zoneType,
    #       const char *region,
    #       const int32_t *rawOffset,
    #       UErrorCode &ec
    # )
    it5 = TimeZone.create_time_zone_id_enumeration(
        USystemTimeZoneType.UCAL_ZONE_TYPE_ANY,
        None,
        None)
    assert isinstance(it5, StringEnumeration)
    assert len(it5) > 0

    it6 = TimeZone.create_time_zone_id_enumeration(
        USystemTimeZoneType.UCAL_ZONE_TYPE_ANY,
        "US",
        None)
    assert isinstance(it6, StringEnumeration)
    assert len(it6) > 0

    raw_offset = -8 * HOUR
    it7 = TimeZone.create_time_zone_id_enumeration(
        USystemTimeZoneType.UCAL_ZONE_TYPE_ANY,
        None,
        raw_offset)
    assert isinstance(it7, StringEnumeration)
    assert len(it7) > 0

    it8 = TimeZone.create_time_zone_id_enumeration(
        USystemTimeZoneType.UCAL_ZONE_TYPE_ANY,
        "US",
        raw_offset)
    assert isinstance(it8, StringEnumeration)
    assert len(it8) > 0

    assert len(it5) > len(it6) > len(it7) > len(it8)
    assert "America/Los_Angeles" in it8

    # [1]
    # static UnicodeString &icu::TimeZone::getCanonicalID(
    #       const UnicodeString &id,
    #       UnicodeString &canonicalID,
    #       UBool &isSystemID,
    #       UErrorCode &status
    # )
    canonical_id = UnicodeString()
    result, is_system_id = TimeZone.get_canonical_id(id_, canonical_id)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(canonical_id)
    assert canonical_id == "America/Los_Angeles"
    assert is_system_id

    canonical_id.remove()
    result, is_system_id = TimeZone.get_canonical_id("PST", canonical_id)
    assert id(result) == id(canonical_id)
    assert canonical_id == "America/Los_Angeles"
    assert is_system_id

    locale = Locale.get_english()

    # [1]
    # UnicodeString &icu::TimeZone::getDisplayName(
    #       const Locale &locale,
    #       UnicodeString &result
    # )
    result = UnicodeString()
    output = zone2.get_display_name(locale, result)
    assert isinstance(output, UnicodeString)
    assert id(output) == id(result)
    assert result == "Pacific Standard Time"
    assert zone2.get_display_name("en", result) == "Pacific Standard Time"

    # [2]
    # UnicodeString &icu::TimeZone::getDisplayName(
    #       UBool inDaylight,
    #       EDisplayType style,
    #       const Locale &locale,
    #       UnicodeString &result
    # )
    output = zone2.get_display_name(True, TimeZone.LONG, locale, result)
    assert isinstance(output, UnicodeString)
    assert id(output) == id(result)
    assert result == "Pacific Daylight Time"
    assert (zone2.get_display_name(True, TimeZone.LONG, "en", result)
            == "Pacific Daylight Time")

    # [3]
    # UnicodeString &icu::TimeZone::getDisplayName(
    #       UBool inDaylight,
    #       EDisplayType style,
    #       UnicodeString &result
    # )
    output = zone2.get_display_name(True, TimeZone.LONG_GMT, result)
    assert isinstance(output, UnicodeString)
    assert id(output) == id(result)
    assert result == "GMT-07:00"

    # [4]
    # UnicodeString &icu::TimeZone::getDisplayName(UnicodeString &result)
    output = zone2.get_display_name(result)
    assert isinstance(output, UnicodeString)
    assert id(output) == id(result)
    assert len(result) > 0

    # UnicodeString &icu::TimeZone::getID(UnicodeString &ID)
    id2 = UnicodeString()
    result = zone2.get_id(id2)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(id2)
    assert id2 == "PST"

    # void icu::TimeZone::setID(const UnicodeString &ID)
    zone2.set_id(UnicodeString("America/Los_Angeles"))
    zone2.get_id(id2)
    assert id2 == "America/Los_Angeles"

    zone2.set_id("US(PST)")
    zone2.get_id(id2)
    assert id2 == "US(PST)"

    # [1]
    # static int32_t icu::TimeZone::getRegion(
    #       const UnicodeString &id,
    #       char *region,
    #       int32_t capacity,
    #       UErrorCode &status
    # )
    region = TimeZone.get_region(id_)
    assert isinstance(region, str)
    assert region == "US"
    assert TimeZone.get_region("PST") == "US"

    # static const char *icu::TimeZone::getTZDataVersion(UErrorCode &status)
    result = TimeZone.get_tz_data_version()
    assert isinstance(result, str)
    assert len(result) > 0  # e.g., "2020d"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 55, reason="ICU4C<55")
def test_time_zone_detect_host_time_zone():
    # static TimeZone *icu::TimeZone::detectHostTimeZone()
    zone = TimeZone.detect_host_time_zone()
    assert isinstance(zone, BasicTimeZone)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 49, reason="ICU4C<49")
def test_time_zone_get_unknown():
    # static const TimeZone &icu::TimeZone::getUnknown()
    zone = TimeZone.get_unknown()
    assert isinstance(zone, BasicTimeZone)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 52, reason="ICU4C<52")
def test_time_zone_get_windows_id():
    # static UnicodeString &icu::TimeZone::getWindowsID(
    #       const UnicodeString &id,
    #       UnicodeString &winid,
    #       UErrorCode &status
    # )
    id_ = UnicodeString("America/New_York")
    winid = UnicodeString()
    result = TimeZone.get_windows_id(id_, winid)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(winid)
    assert winid == "Eastern Standard Time"
    assert (TimeZone.get_windows_id("America/New_York", winid)
            == "Eastern Standard Time")

    # static UnicodeString &icu::TimeZone::getIDForWindowsID(
    #       const UnicodeString &winid,
    #       const char *region,
    #       UnicodeString &id,
    #       UErrorCode &status
    # )
    id_ = UnicodeString()
    result = TimeZone.get_id_for_windows_id(winid, None, id_)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(id_)
    assert id_ == "America/New_York"
    assert (TimeZone.get_id_for_windows_id("Eastern Standard Time", None, id_)
            == "America/New_York")

    id_ = UnicodeString()
    result = TimeZone.get_id_for_windows_id(winid, "CA", id_)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(id_)
    assert id_ == "America/Toronto"
    assert (TimeZone.get_id_for_windows_id("Eastern Standard Time", "CA", id_)
            == "America/Toronto")


def test_time_zone_has_same_rules():
    # UBool icu::TimeZone::hasSameRules(const TimeZone &other)
    id_ = "Europe/Moscow"
    otz = TimeZone.create_time_zone(id_)
    vtz1 = VTimeZone.create_vtime_zone_by_id(id_)
    assert not otz.has_same_rules(vtz1)
    assert vtz1.has_same_rules(otz)

    stz1 = SimpleTimeZone(28800000, "Asia/Singapore")
    stz1.set_start_year(1970)
    stz1.set_start_rule(0, 1, 0)
    stz1.set_end_rule(1, 1, 0)
    vtz2 = VTimeZone.create_vtime_zone_from_basic_time_zone(stz1)
    stz2 = stz1.clone()
    assert vtz2.has_same_rules(stz2)


def test_vtime_zone():
    assert issubclass(VTimeZone, BasicTimeZone)
    assert issubclass(VTimeZone, TimeZone)

    # static VTimeZone *icu::VTimeZone::createVTimeZoneByID(
    #       const UnicodeString &ID
    # )
    id1 = UnicodeString("America/New_York")
    zone1 = VTimeZone.create_vtime_zone_by_id(id1)
    assert isinstance(zone1, VTimeZone)
    zone1.set_tzurl(UnicodeString("http://source.icu-project.org/timezone"))
    zone1.set_last_modified(1215298800000.0)  # 2008-07-05T23:00:00Z
    vtzdata = UnicodeString()
    zone1.write(vtzdata)

    # static VTimeZone *icu::VTimeZone::createVTimeZone(
    #       const UnicodeString &vtzdata,
    #       UErrorCode &status
    # )
    zone2 = VTimeZone.create_vtime_zone(vtzdata)
    assert isinstance(zone2, VTimeZone)

    # static VTimeZone *icu::VTimeZone::createVTimeZoneFromBasicTimeZone(
    #       const BasicTimeZone &basicTZ,
    #       UErrorCode &status
    # )
    basic_tz = SimpleTimeZone(
        0,
        "s3",
        UCalendarMonths.UCAL_JANUARY,
        1,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        0,
        UCalendarMonths.UCAL_JULY,
        1,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        0)
    zone3 = VTimeZone.create_vtime_zone_from_basic_time_zone(basic_tz)
    assert isinstance(zone3, VTimeZone)

    # icu::VTimeZone::VTimeZone(const VTimeZone &source)
    zone4 = VTimeZone(zone3)

    # VTimeZone *icu::VTimeZone::clone()
    zone1a = zone1.clone()
    assert isinstance(zone1a, VTimeZone)
    assert zone1a == zone1

    zone1b = copy.copy(zone1)
    assert zone1b == zone1

    zone1c = copy.deepcopy(zone1)
    assert zone1c == zone1

    # UBool icu::TimeZone::operator!=(const TimeZone &that)
    assert zone1 != zone2
    assert zone1 != zone3
    assert zone1 != zone4
    assert zone2 != zone3
    assert zone2 != zone4
    assert not (zone3 != zone4)

    # UBool icu::VTimeZone::operator==(const TimeZone &that)
    assert not (zone1 == zone2)
    assert not (zone1 == zone3)
    assert not (zone1 == zone4)
    assert not (zone2 == zone3)
    assert not (zone2 == zone4)
    assert zone3 == zone4

    # int32_t icu::BasicTimeZone::countTransitionRules(UErrorCode &status)
    assert zone1.count_transition_rules() == 4
    assert zone2.count_transition_rules() == 17
    assert zone3.count_transition_rules() == 2
    assert zone4.count_transition_rules() == 2

    # int32_t icu::TimeZone::getDSTSavings()
    assert zone1.get_dst_savings() == 1 * HOUR
    assert zone2.get_dst_savings() == 1 * HOUR
    assert zone3.get_dst_savings() == 1 * HOUR

    # UnicodeString &icu::TimeZone::getID(UnicodeString &ID)
    id_ = UnicodeString()
    result = zone1.get_id(id_)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(id_)
    assert len(result) == 0

    # UBool icu::VTimeZone::getLastModified(UDate &lastModified)
    assert zone1.get_last_modified() == (True, 1215298800000.0)
    assert zone2.get_last_modified() == (True, 1215298800000.0)
    result, _ = zone3.get_last_modified()
    assert not result

    # UBool icu::BasicTimeZone::getNextTransition(
    #       UDate base,
    #       UBool inclusive,
    #       TimeZoneTransition &result
    # )
    base = 1230681600000.0  # 2008-12-31T00:00:00Z
    tzt = TimeZoneTransition()
    assert zone3.get_next_transition(base, False, tzt)
    assert tzt.get_time() == 1231027200000.0  # 2009-01-04T00:00:00Z

    # UBool icu::BasicTimeZone::getPreviousTransition(
    #       UDate base,
    #       UBool inclusive,
    #       TimeZoneTransition &result
    # )
    assert zone3.get_previous_transition(base, False, tzt)
    assert tzt.get_time() == 1215298800000.0  # 2008-07-05T23:00:00Z

    # int32_t icu::TimeZone::getRawOffset(void)
    assert zone1.get_raw_offset() == -5 * HOUR
    assert zone2.get_raw_offset() == -5 * HOUR
    assert zone3.get_raw_offset() == 0

    # void icu::BasicTimeZone::getSimpleRulesNear(
    #       UDate date,
    #       InitialTimeZoneRule *&initial,
    #       AnnualTimeZoneRule *&std,
    #       AnnualTimeZoneRule *&dst,
    #       UErrorCode &status
    # )
    initial, std, dst = zone1.get_simple_rules_near(0)
    assert isinstance(initial, InitialTimeZoneRule)
    assert initial.get_raw_offset() == -5 * HOUR
    assert initial.get_dst_savings() == 1 * HOUR
    assert isinstance(std, AnnualTimeZoneRule)
    assert std.get_raw_offset() == -5 * HOUR
    assert std.get_dst_savings() == 0
    dr = std.get_rule()
    assert dr.get_rule_month() == UCalendarMonths.UCAL_OCTOBER
    assert dr.get_rule_day_of_month() == 0
    assert isinstance(dst, AnnualTimeZoneRule)
    assert dst.get_raw_offset() == -5 * HOUR
    assert dst.get_dst_savings() == 1 * HOUR
    dr = dst.get_rule()
    assert dr.get_rule_month() == UCalendarMonths.UCAL_APRIL
    assert dr.get_rule_day_of_month() == 0

    # void icu::BasicTimeZone::getTimeZoneRules(
    #       const InitialTimeZoneRule *&initial,
    #       const TimeZoneRule *trsrules[],
    #       int32_t &trscount,
    #       UErrorCode &status
    # )
    initial, trsrules = zone3.get_time_zone_rules()
    assert isinstance(initial, InitialTimeZoneRule)
    assert initial.get_raw_offset() == 0
    assert initial.get_dst_savings() == 0
    assert isinstance(trsrules, list)
    assert len(trsrules) == 2
    assert all(isinstance(x, AnnualTimeZoneRule) for x in trsrules)
    assert trsrules[0].get_raw_offset() == 0
    assert trsrules[0].get_dst_savings() == 0
    dr = trsrules[0].get_rule()
    assert dr.get_rule_month() == UCalendarMonths.UCAL_JULY
    assert dr.get_rule_day_of_month() == 0
    assert trsrules[1].get_raw_offset() == 0
    assert trsrules[1].get_dst_savings() == 1 * HOUR
    dr = trsrules[1].get_rule()
    assert dr.get_rule_month() == UCalendarMonths.UCAL_JANUARY
    assert dr.get_rule_day_of_month() == 0

    # UBool icu::VTimeZone::getTZURL(UnicodeString &url)
    url = UnicodeString()
    assert zone1.get_tzurl(url)
    assert url == "http://source.icu-project.org/timezone"
    assert not zone3.get_tzurl(url)

    # UBool icu::BasicTimeZone::hasEquivalentTransitions(
    #       const BasicTimeZone &tz,
    #       UDate start,
    #       UDate end,
    #       UBool ignoreDstAmount,
    #       UErrorCode &ec
    # )
    start = 1215298800000.0  # 2008-07-05T23:00:00Z
    end = 1231027200000.0  # 2009-01-04T00:00:00Z
    assert not zone1.has_equivalent_transitions(zone3, start, end, False)
    assert not zone3.has_equivalent_transitions(zone1, start, end, False)

    # UBool icu::TimeZone::hasSameRules(const TimeZone &other)
    assert not zone1.has_same_rules(zone2)
    assert not zone2.has_same_rules(zone3)

    # void icu::TimeZone::setID(const UnicodeString &ID)
    zone1.set_id(UnicodeString("abc"))
    assert zone1.get_id(id_) == "abc"
    zone1.set_id("ABC")
    assert zone1.get_id(id_) == "ABC"

    # void icu::TimeZone::setRawOffset(int32_t offsetMillis)
    zone1.set_raw_offset(5 * HOUR)  # Nothing to do
    assert zone1.get_raw_offset() == -5 * HOUR

    # void icu::VTimeZone::setTZURL(const UnicodeString &url)
    zone1.set_tzurl("http://www.example.com")
    assert zone1.get_tzurl(url)
    assert url == "http://www.example.com"

    # UBool icu::TimeZone::useDaylightTime(void)
    assert zone1.use_daylight_time()
    assert zone2.use_daylight_time()
    assert zone3.use_daylight_time()

    # [1]
    # void icu::VTimeZone::write(
    #       UDate start,
    #       UnicodeString &result,
    #       UErrorCode &status
    # )
    result1 = UnicodeString()
    zone1.write(1231027200000.0, result1)
    assert len(result1) > 0

    # [2]
    # void icu::VTimeZone::write(
    #       UnicodeString &result,
    #       UErrorCode &status
    # )
    result2 = UnicodeString()
    zone1.write(result2)
    assert len(result2) > 0

    # void icu::VTimeZone::writeSimple(
    #       UDate time,
    #       UnicodeString &result,
    #       UErrorCode &status
    # )
    result3 = UnicodeString()
    zone1.write_simple(1231027200000.0, result3)
    assert len(result3) > 0


def test_vtime_zone_get_offset():
    basic_tz = SimpleTimeZone(
        8 * HOUR,
        "s5",
        UCalendarMonths.UCAL_APRIL,
        1,
        -1,
        2 * HOUR,
        SimpleTimeZone.WALL_TIME,
        UCalendarMonths.UCAL_OCTOBER,
        -1,
        1,
        2 * HOUR,
        SimpleTimeZone.WALL_TIME,
        int(0.5 * HOUR))
    zone = VTimeZone.create_vtime_zone_from_basic_time_zone(basic_tz)

    # [1]
    # void icu::VTimeZone::getOffset(
    #       UDate date,
    #       UBool local,
    #       int32_t &rawOffset,
    #       int32_t &dstOffset,
    #       UErrorCode &ec
    # )
    raw_offset, dst_offset = zone.get_offset(0, False)
    assert raw_offset == 8 * HOUR
    assert dst_offset == 0

    # [2]
    # int32_t icu::VTimeZone::getOffset(
    #       uint8_t era,
    #       int32_t year,
    #       int32_t month,
    #       int32_t day,
    #       uint8_t dayOfWeek,
    #       int32_t millis,
    #       int32_t monthLength,
    #       UErrorCode &status
    # )
    assert zone.get_offset(
        GregorianCalendar.AD,
        2008,
        UCalendarMonths.UCAL_JULY,
        6,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        0,
        28) == 8.5 * HOUR

    # [3]
    # int32_t icu::VTimeZone::getOffset(
    #       uint8_t era,
    #       int32_t year,
    #       int32_t month,
    #       int32_t day,
    #       uint8_t dayOfWeek,
    #       int32_t millis,
    #       UErrorCode &status
    # )
    assert zone.get_offset(
        GregorianCalendar.AD,
        2008,
        UCalendarMonths.UCAL_JULY,
        6,
        UCalendarDaysOfWeek.UCAL_SUNDAY,
        0) == 8.5 * HOUR


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 69, reason="ICU4C<69")
def test_vtime_zone_get_offset_from_local():
    from icupy import UTimeZoneLocalOption

    # From icu/source/test/intltest/tzoffloc.cpp:
    # TimeZoneOffsetLocalTest::TestGetOffsetAroundTransition()
    zone = VTimeZone.create_vtime_zone_by_id("America/Los_Angeles")

    # void icu::VTimeZone::getOffsetFromLocal(
    #       UDate date,
    #       UTimeZoneLocalOption nonExistingTimeOpt,
    #       UTimeZoneLocalOption duplicatedTimeOpt,
    #       int32_t &rawOffset,
    #       int32_t &dstOffset,
    #       UErrorCode &status
    # )
    date = 1143941400000  # 2006-04-02T01:30:00Z
    raw_offset, dst_offset = zone.get_offset_from_local(
        date,
        UTimeZoneLocalOption.UCAL_TZ_LOCAL_STANDARD_FORMER,
        UTimeZoneLocalOption.UCAL_TZ_LOCAL_STANDARD_LATTER)
    assert raw_offset == -8 * HOUR
    assert dst_offset == 0



import pytest
from icupy import U_ICU_VERSION_MAJOR_NUM

if U_ICU_VERSION_MAJOR_NUM < 50:
    pytest.skip("ICU4C<50", allow_module_level=True)

import copy

from icupy import (
    FieldPosition, FieldPositionIterator, Formattable, ICUError, Locale,
    NumberFormat, ParsePosition, PluralFormat, PluralRules, UErrorCode,
    UPluralType, UnicodeString,
)


def test_api():
    rules = PluralRules.create_rules("odd: n mod 2 is 1")
    fmt = PluralFormat(rules)

    # void icu::PluralFormat::applyPattern(
    #       const UnicodeString &pattern,
    #       UErrorCode &status
    # )
    pattern = UnicodeString("odd {# is odd.} other{# is even.}")
    fmt.apply_pattern(pattern)

    fmt.apply_pattern(str(pattern))

    # UnicodeString &icu::PluralFormat::toPattern(UnicodeString &appendTo)
    append_to = UnicodeString()
    result = fmt.to_pattern(append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == pattern

    # void icu::PluralFormat::setNumberFormat(
    #       const NumberFormat *format,
    #       UErrorCode &status
    # )
    numfmt = NumberFormat.create_instance()
    fmt.set_number_format(numfmt)

    # UBool icu::PluralFormat::operator!=(const Format &other)
    fmt1 = PluralFormat(Locale("en"), rules)
    fmt2 = PluralFormat(Locale("en"), rules)
    fmt3 = PluralFormat(Locale("fr"), rules)
    assert not (fmt1 != fmt2)
    assert fmt1 != fmt3
    assert fmt2 != fmt3

    # UBool icu::PluralFormat::operator==(const Format &other)
    assert fmt1 == fmt2
    assert not (fmt1 == fmt3)
    assert not (fmt2 == fmt3)


def test_clone():
    fmt1 = PluralFormat(Locale.get_english())

    # PluralFormat *icu::PluralFormat::clone()
    fmt2 = fmt1.clone()
    assert isinstance(fmt2, PluralFormat)
    assert fmt1 == fmt2

    fmt3 = copy.copy(fmt1)
    assert fmt1 == fmt3

    fmt4 = copy.deepcopy(fmt1)
    assert fmt1 == fmt4


def test_format():
    # From icu/source/test/intltest/plurfmts.cpp
    pattern = "one{#st file}two{#nd file}few{#rd file}other{#th file}"
    fmt = PluralFormat(
        Locale.get_english(),
        UPluralType.UPLURAL_TYPE_ORDINAL,
        pattern)
    append_to = UnicodeString()

    # [1], [2]
    # UnicodeString &icu::PluralFormat::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       FieldPosition &pos,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(Formattable(3), append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "3rd file"

    # *U_UNSUPPORTED_ERROR in ICU 69*
    # [3]
    # UnicodeString &icu::Format::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       FieldPositionIterator *posIter,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos_iter = FieldPositionIterator()
    with pytest.raises(ICUError) as exc_info:
        _ = fmt.format(Formattable(3), append_to, pos_iter)
    assert exc_info.value.args[0] == UErrorCode.U_UNSUPPORTED_ERROR

    append_to.remove()
    with pytest.raises(ICUError) as exc_info:
        _ = fmt.format(Formattable(3), append_to, None)
    assert exc_info.value.args[0] == UErrorCode.U_UNSUPPORTED_ERROR

    # [4]
    # UnicodeString &icu::Format::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       UErrorCode &status
    # )
    append_to.remove()
    result = fmt.format(Formattable(3), append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "3rd file"

    # [5]
    # UnicodeString icu::PluralFormat::format(
    #       double number,
    #       UErrorCode &status
    # )
    result = fmt.format(3.0)
    assert isinstance(result, UnicodeString)
    assert result == "3rd file"

    # [6]
    # UnicodeString &icu::PluralFormat::format(
    #       double number,
    #       UnicodeString &appendTo,
    #       FieldPosition &pos,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(3.0, append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "3rd file"

    # [7]
    # UnicodeString icu::PluralFormat::format(
    #       int32_t number,
    #       UErrorCode &status
    # )
    result = fmt.format(3)
    assert isinstance(result, UnicodeString)
    assert result == "3rd file"

    # [8]
    # UnicodeString &icu::PluralFormat::format(
    #       int32_t number,
    #       UnicodeString &appendTo,
    #       FieldPosition &pos,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(3, append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "3rd file"


def test_parse_object():
    pattern = "one{#st file}two{#nd file}few{#rd file}other{#th file}"
    fmt = PluralFormat(
        Locale.get_english(),
        UPluralType.UPLURAL_TYPE_ORDINAL,
        pattern)

    # *No effect in ICU 69*
    # void icu::PluralFormat::parseObject(
    #       const UnicodeString &source,
    #       Formattable &result,
    #       ParsePosition &parse_pos
    # )
    source = UnicodeString("3rd file")
    result = Formattable()
    parse_pos = ParsePosition()
    fmt.parse_object(source, result, parse_pos)
    assert parse_pos.get_index() == 0

    result = Formattable()
    parse_pos = ParsePosition()
    fmt.parse_object(str(source), result, parse_pos)
    assert parse_pos.get_index() == 0

    # *U_INVALID_FORMAT_ERROR in ICU 69*
    # void icu::Format::parseObject(
    #       const UnicodeString &source,
    #       Formattable &result,
    #       UErrorCode &status
    # )
    result = Formattable()
    with pytest.raises(ICUError) as exc_info:
        fmt.parse_object(source, result)
    assert exc_info.value.args[0] == UErrorCode.U_INVALID_FORMAT_ERROR

    result = Formattable()
    with pytest.raises(ICUError) as exc_info:
        fmt.parse_object(str(source), result)
    assert exc_info.value.args[0] == UErrorCode.U_INVALID_FORMAT_ERROR


def test_plural_format():
    locale = Locale.get_default()
    rules = PluralRules.create_default_rules()
    pattern = UnicodeString("other{#}")

    # [1]
    # icu::PluralFormat::PluralFormat(UErrorCode &status)
    fmt1 = PluralFormat()

    # [2]
    # icu::PluralFormat::PluralFormat(
    #       const Locale &locale,
    #       UErrorCode &status
    # )
    fmt2 = PluralFormat(locale)
    assert fmt1 == fmt2

    # [3]
    # icu::PluralFormat::PluralFormat(
    #       const PluralRules &rules,
    #       UErrorCode &status
    # )
    fmt3 = PluralFormat(rules)

    # [4]
    # icu::PluralFormat::PluralFormat(
    #       const Locale &locale,
    #       const PluralRules &rules,
    #       UErrorCode &status
    # )
    fmt4 = PluralFormat(locale, rules)
    assert fmt3 == fmt4

    fmt4a = PluralFormat(str(locale), rules)
    assert fmt4 == fmt4a

    # [5]
    # icu::PluralFormat::PluralFormat(
    #       const Locale &locale,
    #       UPluralType type,
    #       UErrorCode &status
    # )
    fmt5 = PluralFormat(locale, UPluralType.UPLURAL_TYPE_CARDINAL)

    fmt5a = PluralFormat(str(locale), UPluralType.UPLURAL_TYPE_CARDINAL)
    assert fmt5 == fmt5a

    # [6]
    # icu::PluralFormat::PluralFormat(
    #       const UnicodeString &pattern,
    #       UErrorCode &status
    # )
    fmt6 = PluralFormat(pattern)

    fmt6a = PluralFormat(str(pattern))
    assert fmt6 == fmt6a

    # [7]
    # icu::PluralFormat::PluralFormat(
    #       const Locale &locale,
    #       const UnicodeString &pattern,
    #       UErrorCode &status
    # )
    fmt7 = PluralFormat(locale, pattern)
    assert fmt6 == fmt7

    fmt7a = PluralFormat(str(locale), pattern)
    assert fmt7 == fmt7a

    fmt7b = PluralFormat(locale, str(pattern))
    assert fmt7 == fmt7b

    fmt7c = PluralFormat(str(locale), str(pattern))
    assert fmt7 == fmt7c

    # [8]
    # icu::PluralFormat::PluralFormat(
    #       const PluralRules &rules,
    #       const UnicodeString &pattern,
    #       UErrorCode &status
    # )
    fmt8 = PluralFormat(rules, pattern)

    fmt8a = PluralFormat(rules, str(pattern))
    assert fmt8 == fmt8a

    # [9]
    # icu::PluralFormat::PluralFormat(
    #       const Locale &locale,
    #       const PluralRules &rules,
    #       const UnicodeString &pattern,
    #       UErrorCode &status
    # )
    fmt9 = PluralFormat(locale, rules, pattern)

    fmt9a = PluralFormat(str(locale), rules, pattern)
    assert fmt9 == fmt9a

    fmt9b = PluralFormat(locale, rules, str(pattern))
    assert fmt9 == fmt9b

    fmt9c = PluralFormat(str(locale), rules, str(pattern))
    assert fmt9 == fmt9c

    # [10]
    # icu::PluralFormat::PluralFormat(
    #       const Locale &locale,
    #       UPluralType type,
    #       const UnicodeString &pattern,
    #       UErrorCode &status
    # )
    fmt10 = PluralFormat(
        locale,
        UPluralType.UPLURAL_TYPE_CARDINAL,
        pattern)

    fmt10a = PluralFormat(
        str(locale),
        UPluralType.UPLURAL_TYPE_CARDINAL,
        pattern)
    assert fmt10 == fmt10a

    fmt10b = PluralFormat(
        locale,
        UPluralType.UPLURAL_TYPE_CARDINAL,
        str(pattern))
    assert fmt10 == fmt10b

    fmt10c = PluralFormat(
        str(locale),
        UPluralType.UPLURAL_TYPE_CARDINAL,
        str(pattern))
    assert fmt10 == fmt10c

    # [11]
    # icu::PluralFormat::PluralFormat(const PluralFormat &other)
    fmt11 = PluralFormat(fmt1)
    assert fmt1 == fmt11



from functools import partial

from icupy import (
    ucsdet_close, ucsdet_detect, ucsdet_detect_all,
    ucsdet_enable_input_filter, ucsdet_get_all_detectable_charsets,
    ucsdet_get_confidence, ucsdet_get_language, ucsdet_get_name,
    ucsdet_get_uchars, ucsdet_is_input_filter_enabled, ucsdet_open,
    ucsdet_set_declared_encoding, ucsdet_set_text,
    uenum_close, uenum_count, uenum_next
)

from . import gc


def test_api():
    with gc(ucsdet_open(), ucsdet_close) as ucsd:
        en = ucsdet_get_all_detectable_charsets(ucsd)
        assert uenum_count(en) > 0
        charsets = iter(partial(uenum_next, en), None)
        assert "UTF-8" in charsets
        uenum_close(en)

        utf8 = (
            "\u3042\u308b\u65e5\u306e\u66ae\u65b9\u306e\u4e8b\u3067\u3042"
            "\u308b\u3002\u4e00\u4eba\u306e\u4e0b\u4eba\u304c\u3001\u7f85"
            "\u751f\u9580\u306e\u4e0b\u3067\u96e8\u3084\u307f\u3092\u5f85"
            "\u3063\u3066\u3044\u305f\u3002\u5e83\u3044\u9580\u306e\u4e0b"
            "\u306b\u306f\u3001\u3053\u306e\u7537\u306e\u307b\u304b\u306b"
            "\u8ab0\u3082\u3044\u306a\u3044\u3002\u305f\u3060\u3001\u6240"
            "\u3005\u4e39\u5857\u306e\u5265\u3052\u305f\u3001\u5927\u304d"
            "\u306a\u5186\u67f1\u306b\u3001\u87cb\u87c0\u304c\u4e00\u5339"
            "\u3068\u307e\u3063\u3066\u3044\u308b\u3002\u7f85\u751f\u9580"
            "\u304c\u3001\u6731\u96c0\u5927\u8def\u306b\u3042\u308b\u4ee5"
            "\u4e0a\u306f\u3001\u3053\u306e\u7537\u306e\u307b\u304b\u306b"
            "\u3082\u3001\u96e8\u3084\u307f\u3092\u3059\u308b\u5e02\u5973"
            "\u7b20\u3084\u63c9\u70cf\u5e3d\u5b50\u304c\u3001\u3082\u3046"
            "\u4e8c\u4e09\u4eba\u306f\u3042\u308a\u305d\u3046\u306a\u3082"
            "\u306e\u3067\u3042\u308b\u3002\u305d\u308c\u304c\u3001\u3053"
            "\u306e\u7537\u306e\u307b\u304b\u306b\u306f\u8ab0\u3082\u3044"
            "\u306a\u3044\u3002"
        )

        ucsdet_set_text(ucsd, utf8, -1)
        ucsm = ucsdet_detect(ucsd)
        assert ucsm
        assert len(ucsdet_get_language(ucsm)) == 0
        assert ucsdet_get_name(ucsm) == "UTF-8"

        text_in = utf8.encode("shift_jis")
        ucsdet_set_text(ucsd, text_in, len(text_in))
        ucsm = ucsdet_detect(ucsd)
        assert ucsm
        assert ucsdet_get_language(ucsm) == "ja"
        assert ucsdet_get_name(ucsm) == "Shift_JIS"

        matches = ucsdet_detect_all(ucsd)
        assert isinstance(matches, list)
        assert len(matches) > 0
        assert ucsdet_get_confidence(matches[0]) == 100
        if len(matches) >= 2:
            assert (ucsdet_get_confidence(matches[0])
                    > ucsdet_get_confidence(matches[1]))
        assert ucsdet_get_language(matches[0]) == "ja"
        assert ucsdet_get_name(matches[0]) == "Shift_JIS"

        buf = ucsdet_get_uchars(matches[0])
        assert isinstance(buf, str)
        assert buf == utf8

        assert not ucsdet_is_input_filter_enabled(ucsd)
        assert not ucsdet_enable_input_filter(ucsd, True)
        assert ucsdet_is_input_filter_enabled(ucsd)

        ucsdet_set_declared_encoding(ucsd, "UTF-8", -1)



import copy

import pytest
from icupy import (
    Calendar, DateFormat, DateInterval, DateIntervalFormat, DateIntervalInfo,
    FieldPosition, FieldPositionIterator, Format, Formattable, ICUError,
    Locale, TimeZone, UCalendarDateFields, UErrorCode,
    U_ICU_VERSION_MAJOR_NUM, UnicodeString,
)


def test_api():
    assert issubclass(DateIntervalFormat, Format)

    fmt1 = DateIntervalFormat.create_instance("yMMMd", Locale.get_english())
    fmt2 = DateIntervalFormat.create_instance("yMMMd", "en")
    fmt3 = DateIntervalFormat.create_instance("yMMMd", Locale.get_japanese())

    # UBool icu::DateIntervalFormat::operator!=(const Format &)
    assert not (fmt1 != fmt2)
    assert fmt1 != fmt3
    assert fmt2 != fmt3

    # UBool icu::DateIntervalFormat::operator==(const Format &)
    assert fmt1 == fmt2
    assert not (fmt1 == fmt3)
    assert not (fmt2 == fmt3)

    # const DateFormat *icu::DateIntervalFormat::getDateFormat(void)
    dtfmt = fmt2.get_date_format()
    assert isinstance(dtfmt, DateFormat)

    # const DateIntervalInfo *icu::DateIntervalFormat::getDateIntervalInfo()
    dtitvinf = fmt2.get_date_interval_info()
    assert isinstance(dtitvinf, DateIntervalInfo)

    # void icu::DateIntervalFormat::setDateIntervalInfo(
    #       const DateIntervalInfo &newIntervalPatterns,
    #       UErrorCode &status
    # )
    new_interval_patterns = DateIntervalInfo(Locale.get_japanese())
    assert new_interval_patterns != dtitvinf
    fmt2.set_date_interval_info(new_interval_patterns)
    assert fmt2.get_date_interval_info() == new_interval_patterns

    # const TimeZone &icu::DateIntervalFormat::getTimeZone(void)
    zone2 = fmt2.get_time_zone()
    assert isinstance(zone2, TimeZone)

    # void icu::DateIntervalFormat::setTimeZone(const TimeZone &zone)
    zone = TimeZone.create_time_zone("JST")
    assert zone != zone2
    fmt2.set_time_zone(zone)
    assert fmt2.get_time_zone() == zone


def test_clone():
    fmt1 = DateIntervalFormat.create_instance("yMMMd", Locale.get_english())

    # DateIntervalFormat *icu::DateIntervalFormat::clone()
    fmt2 = fmt1.clone()
    assert isinstance(fmt2, DateIntervalFormat)
    assert fmt2 == fmt1

    fmt3 = copy.copy(fmt1)
    assert fmt3 == fmt1

    fmt4 = copy.deepcopy(fmt1)
    assert fmt4 == fmt1


def test_create_instance():
    dtitvinf = DateIntervalInfo(Locale.get_english())

    # [1]
    # static DateIntervalFormat *icu::DateIntervalFormat::createInstance(
    #       const UnicodeString &skeleton,
    #       const DateIntervalInfo &dtitvinf,
    #       UErrorCode &status
    # )
    fmt1 = DateIntervalFormat.create_instance(
        UnicodeString("yMMMd"),
        dtitvinf)
    assert isinstance(fmt1, DateIntervalFormat)

    fmt1a = DateIntervalFormat.create_instance(
        "yMMMd",
        dtitvinf)
    assert isinstance(fmt1a, DateIntervalFormat)
    assert fmt1 == fmt1a

    # [2]
    # static DateIntervalFormat *icu::DateIntervalFormat::createInstance(
    #       const UnicodeString &skeleton,
    #       const Locale &locale,
    #       const DateIntervalInfo &dtitvinf,
    #       UErrorCode &status
    # )
    fmt2 = DateIntervalFormat.create_instance(
        UnicodeString("yMMMd"),
        Locale("en"),
        dtitvinf)
    assert isinstance(fmt2, DateIntervalFormat)

    fmt2a = DateIntervalFormat.create_instance(
        "yMMMd",
        Locale("en"),
        dtitvinf)
    assert isinstance(fmt2a, DateIntervalFormat)

    fmt2b = DateIntervalFormat.create_instance(
        UnicodeString("yMMMd"),
        "en",
        dtitvinf)
    assert isinstance(fmt2b, DateIntervalFormat)

    fmt2c = DateIntervalFormat.create_instance(
        "yMMMd",
        "en",
        dtitvinf)
    assert isinstance(fmt2c, DateIntervalFormat)
    assert fmt2 == fmt2a == fmt2b == fmt2c

    # [3]
    # static DateIntervalFormat *icu::DateIntervalFormat::createInstance(
    #       const UnicodeString &skeleton,
    #       const Locale &locale,
    #       UErrorCode &status
    # )
    fmt3 = DateIntervalFormat.create_instance(
        UnicodeString("yMMMd"),
        Locale("en"))
    assert isinstance(fmt3, DateIntervalFormat)

    fmt3a = DateIntervalFormat.create_instance(
        "yMMMd",
        Locale("en"))
    assert isinstance(fmt3a, DateIntervalFormat)

    fmt3b = DateIntervalFormat.create_instance(
        UnicodeString("yMMMd"),
        "en")
    assert isinstance(fmt3b, DateIntervalFormat)

    fmt3c = DateIntervalFormat.create_instance(
        "yMMMd",
        "en")
    assert isinstance(fmt3c, DateIntervalFormat)
    assert fmt3 == fmt3a == fmt3b == fmt3c

    # [4]
    # static DateIntervalFormat *icu::DateIntervalFormat::createInstance(
    #       const UnicodeString &skeleton,
    #       UErrorCode &status
    # )
    fmt4 = DateIntervalFormat.create_instance(UnicodeString("yMMMd"))
    assert isinstance(fmt4, DateIntervalFormat)

    fmt4a = DateIntervalFormat.create_instance("yMMMd")
    assert isinstance(fmt4a, DateIntervalFormat)
    assert fmt4 == fmt4a


def test_date_interval():
    from_date = 1366934400000.0  # 2013-04-26T00:00:00Z
    to_date = 1367107200000.0  # 2013-04-28T00:00:00Z

    # [1]
    # icu::DateInterval::DateInterval(
    #       UDate fromDate,
    #       UDate toDate
    # )
    itv1 = DateInterval(from_date, to_date)

    # [2]
    # icu::DateInterval::DateInterval(const DateInterval &other)
    itv2 = DateInterval(itv1)

    # DateInterval *icu::DateInterval::clone()
    itv3 = itv1.clone()
    assert isinstance(itv3, DateInterval)

    # UBool icu::DateInterval::operator!=(const DateInterval &)
    assert not (itv1 != itv2)
    assert not (itv1 != itv3)
    assert not (itv2 != itv3)

    # UBool icu::DateInterval::operator==(const DateInterval &)
    assert itv1 == itv2 == itv3

    # UDate icu::DateInterval::getFromDate()
    assert itv2.get_from_date() == from_date

    # UDate icu::DateInterval::getToDate()
    assert itv2.get_to_date() == to_date


def test_date_interval_info():
    # [2]
    # icu::DateIntervalInfo::DateIntervalInfo(
    #       const Locale &locale,
    #       UErrorCode &status
    # )
    dtitvinf2 = DateIntervalInfo(Locale("en"))
    dtitvinf2a = DateIntervalInfo("en")
    dtitvinf2b = DateIntervalInfo(Locale("ja"))

    # [3]
    # icu::DateIntervalInfo::DateIntervalInfo(const DateIntervalInfo &)
    dtitvinf3 = DateIntervalInfo(dtitvinf2)

    # icu::DateIntervalInfo *DateIntervalInfo::clone()
    dtitvinf4 = dtitvinf2.clone()
    assert isinstance(dtitvinf4, DateIntervalInfo)

    # UBool icu::DateIntervalInfo::operator!=(const DateIntervalInfo &)
    assert not (dtitvinf2 != dtitvinf2a)
    assert dtitvinf2 != dtitvinf2b
    assert not (dtitvinf2 != dtitvinf3)
    assert not (dtitvinf2 != dtitvinf4)

    # UBool icu::DateIntervalInfo::operator==(const DateIntervalInfo &)
    assert dtitvinf2 == dtitvinf2a == dtitvinf3 == dtitvinf4
    assert not (dtitvinf2 == dtitvinf2b)

    # UBool icu::DateIntervalInfo::getDefaultOrder()
    assert not dtitvinf2.get_default_order()

    # UnicodeString &icu::DateIntervalInfo::getFallbackIntervalPattern(
    #       UnicodeString &result
    # )
    result = UnicodeString()
    pattern = dtitvinf3.get_fallback_interval_pattern(result)
    assert isinstance(pattern, UnicodeString)
    assert id(result) == id(pattern)
    assert result == "{0} \u2013 {1}"

    # void icu::DateIntervalInfo::setFallbackIntervalPattern(
    #       const UnicodeString &fallbackPattern,
    #       UErrorCode &status
    # )
    dtitvinf3.set_fallback_interval_pattern(UnicodeString("{0} ~ {1}"))
    assert dtitvinf3.get_fallback_interval_pattern(result) == "{0} ~ {1}"

    dtitvinf3.set_fallback_interval_pattern("{0} \u2013 {1}")
    assert dtitvinf3.get_fallback_interval_pattern(result) == "{0} \u2013 {1}"

    # UnicodeString &icu::DateIntervalInfo::getIntervalPattern(
    #       const UnicodeString &skeleton,
    #       UCalendarDateFields field,
    #       UnicodeString &result,
    #       UErrorCode &status
    # )
    result = UnicodeString()
    pattern = dtitvinf3.get_interval_pattern(
        UnicodeString("Hm"),
        UCalendarDateFields.UCAL_HOUR_OF_DAY,
        result)
    assert isinstance(pattern, UnicodeString)
    assert id(result) == id(pattern)
    assert result == "HH:mm \u2013 HH:mm"

    pattern = dtitvinf3.get_interval_pattern(
        "Hm",
        UCalendarDateFields.UCAL_HOUR_OF_DAY,
        result)
    assert isinstance(pattern, UnicodeString)
    assert id(result) == id(pattern)
    assert result == "HH:mm \u2013 HH:mm"

    # void icu::DateIntervalInfo::setIntervalPattern(
    #       const UnicodeString &skeleton,
    #       UCalendarDateFields lrgDiffCalUnit,
    #       const UnicodeString &intervalPattern,
    #       UErrorCode &status
    # )
    dtitvinf3.set_interval_pattern(
        UnicodeString("Hm"),
        UCalendarDateFields.UCAL_HOUR_OF_DAY,
        UnicodeString("yyyy MMM d HH:mm ~ HH:mm"))
    assert dtitvinf3.get_interval_pattern(
        "Hm",
        UCalendarDateFields.UCAL_HOUR_OF_DAY,
        result) == "yyyy MMM d HH:mm ~ HH:mm"

    dtitvinf3.set_interval_pattern(
        "Hm",
        UCalendarDateFields.UCAL_HOUR_OF_DAY,
        UnicodeString("HH:mm ~ HH:mm"))
    assert dtitvinf3.get_interval_pattern(
        "Hm",
        UCalendarDateFields.UCAL_HOUR_OF_DAY,
        result) == "HH:mm ~ HH:mm"

    dtitvinf3.set_interval_pattern(
        UnicodeString("Hm"),
        UCalendarDateFields.UCAL_HOUR_OF_DAY,
        "yyyy MMM d HH:mm ~ HH:mm")
    assert dtitvinf3.get_interval_pattern(
        "Hm",
        UCalendarDateFields.UCAL_HOUR_OF_DAY,
        result) == "yyyy MMM d HH:mm ~ HH:mm"

    dtitvinf3.set_interval_pattern(
        "Hm",
        UCalendarDateFields.UCAL_HOUR_OF_DAY,
        "HH:mm ~ HH:mm")
    assert dtitvinf3.get_interval_pattern(
        "Hm",
        UCalendarDateFields.UCAL_HOUR_OF_DAY,
        result) == "HH:mm ~ HH:mm"


def test_format():
    en_locale = Locale.get_english()
    fmt = DateIntervalFormat.create_instance("yMMMd", en_locale)
    from_date = 1366934400000.0  # 2013-04-26T00:00:00Z
    to_date = 1367107200000.0  # 2013-04-28T00:00:00Z

    # [1]
    # UnicodeString &icu::DateIntervalFormat::format(
    #       Calendar &fromCalendar,
    #       Calendar &toCalendar,
    #       UnicodeString &appendTo,
    #       FieldPosition &fieldPosition,
    #       UErrorCode &status
    # )
    zone = TimeZone.get_gmt()
    from_calendar = Calendar.create_instance(zone, en_locale)
    from_calendar.set_time(from_date)
    to_calendar = Calendar.create_instance(zone, en_locale)
    to_calendar.set_time(to_date)
    append_to = UnicodeString()
    field_position = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(
        from_calendar,
        to_calendar,
        append_to,
        field_position)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "Apr 26 \u2013 28, 2013"

    # [2]
    # UnicodeString &icu::DateIntervalFormat::format(
    #       const DateInterval *dtInterval,
    #       UnicodeString &appendTo,
    #       FieldPosition &fieldPosition,
    #       UErrorCode &status
    # )
    dt_interval = DateInterval(from_date, to_date)
    append_to = UnicodeString()
    field_position = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(dt_interval, append_to, field_position)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "Apr 26 \u2013 28, 2013"

    # [3]
    # UnicodeString &icu::DateIntervalFormat::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       FieldPosition &fieldPosition,
    #       UErrorCode &status
    # )
    append_to = UnicodeString()
    field_position = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(
        Formattable(DateInterval(from_date, to_date)),
        append_to,
        field_position)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "Apr 26 \u2013 28, 2013"

    # [5]
    # UnicodeString &icu::Format::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       FieldPositionIterator *posIter,
    #       UErrorCode &status
    # )
    append_to = UnicodeString()
    pos_iter = FieldPositionIterator()
    with pytest.raises(ICUError) as exc_info:
        _ = fmt.format(
            Formattable(DateInterval(from_date, to_date)),
            append_to,
            pos_iter)
    assert exc_info.value.args[0] == UErrorCode.U_UNSUPPORTED_ERROR

    # [6]
    # UnicodeString &icu::Format::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       UErrorCode &status
    # )
    append_to = UnicodeString()
    result = fmt.format(
        Formattable(DateInterval(from_date, to_date)),
        append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "Apr 26 \u2013 28, 2013"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 64, reason="ICU4C<64")
def test_format_to_value():
    from icupy import FormattedDateInterval

    from_date = 1366934400000.0  # 2013-04-26T00:00:00Z
    to_date = 1367107200000.0  # 2013-04-28T00:00:00Z

    en_locale = Locale.get_english()
    fmt = DateIntervalFormat.create_instance("yMMMd", en_locale)
    zone = TimeZone.get_gmt()
    from_calendar = Calendar.create_instance(zone, en_locale)
    from_calendar.set_time(from_date)
    to_calendar = Calendar.create_instance(zone, en_locale)
    to_calendar.set_time(to_date)

    # [1]
    # FormattedDateInterval icu::DateIntervalFormat::formatToValue(
    #       Calendar &fromCalendar,
    #       Calendar &toCalendar,
    #       UErrorCode &status
    # )
    dtitv1 = fmt.format_to_value(from_calendar, to_calendar)
    assert isinstance(dtitv1, FormattedDateInterval)
    assert dtitv1.to_temp_string() == "Apr 26 \u2013 28, 2013"

    # [2]
    # FormattedDateInterval icu::DateIntervalFormat::formatToValue(
    #       const DateInterval &dtInterval,
    #       UErrorCode &status
    # )
    dt_interval = DateInterval(from_date, to_date)
    dtitv2 = fmt.format_to_value(dt_interval)
    assert isinstance(dtitv2, FormattedDateInterval)
    assert dtitv2.to_temp_string() == "Apr 26 \u2013 28, 2013"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 64, reason="ICU4C<64")
def test_formatted_date_interval():
    from icupy import (
        ConstrainedFieldPosition, FormattedDateInterval, UDateFormatField,
        UFieldCategory, UnicodeStringAppendable,
    )

    fmt = DateIntervalFormat.create_instance("yMMMd", Locale.get_english())
    from_date = 1366934400000.0  # 2013-04-26T00:00:00Z
    to_date = 1367107200000.0  # 2013-04-28T00:00:00Z
    dtitv = fmt.format_to_value(DateInterval(from_date, to_date))
    assert isinstance(dtitv, FormattedDateInterval)

    # Appendable &icu::FormattedDateInterval::appendTo(
    #       Appendable &appendable,
    #       UErrorCode &status
    # )
    dest = UnicodeString()
    appendable = UnicodeStringAppendable(dest)
    appendable.append_code_point(0x1f338)
    result = dtitv.append_to(appendable)
    assert isinstance(result, UnicodeStringAppendable)
    assert id(result) == id(appendable)
    assert dest == "\U0001f338Apr 26 \u2013 28, 2013"

    # UBool icu::FormattedDateInterval::nextPosition(
    #       ConstrainedFieldPosition &cfpos,
    #       UErrorCode &status
    # )
    cfpos = ConstrainedFieldPosition()
    assert dtitv.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_DATE
    assert cfpos.get_field() == UDateFormatField.UDAT_MONTH_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (0, 3)

    assert dtitv.next_position(cfpos)
    assert (cfpos.get_category()
            == UFieldCategory.UFIELD_CATEGORY_DATE_INTERVAL_SPAN)
    assert cfpos.get_field() == 0

    assert dtitv.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_DATE
    assert cfpos.get_field() == UDateFormatField.UDAT_DATE_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (4, 6)

    assert dtitv.next_position(cfpos)
    assert (cfpos.get_category()
            == UFieldCategory.UFIELD_CATEGORY_DATE_INTERVAL_SPAN)
    assert cfpos.get_field() == 1

    assert dtitv.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_DATE
    assert cfpos.get_field() == UDateFormatField.UDAT_DATE_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (9, 11)

    assert dtitv.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_DATE
    assert cfpos.get_field() == UDateFormatField.UDAT_YEAR_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (13, 17)

    assert not dtitv.next_position(cfpos)

    # UnicodeString icu::FormattedDateInterval::toString(UErrorCode &status)
    result = dtitv.to_string()
    assert isinstance(result, UnicodeString)
    assert result == "Apr 26 \u2013 28, 2013"

    # UnicodeString icu::FormattedDateInterval::toTempString(
    #       UErrorCode &status
    # )
    result = dtitv.to_temp_string()
    assert isinstance(result, UnicodeString)
    assert result == "Apr 26 \u2013 28, 2013"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 68, reason="ICU4C<68")
def test_get_context():
    from icupy import UDisplayContext, UDisplayContextType

    fmt = DateIntervalFormat.create_instance("yMMMd", Locale.get_english())

    # UDisplayContext icu::DateIntervalFormat::getContext(
    #       UDisplayContextType type,
    #       UErrorCode &status
    # )
    assert (fmt.get_context(UDisplayContextType.UDISPCTX_TYPE_CAPITALIZATION)
            == UDisplayContext.UDISPCTX_CAPITALIZATION_NONE)

    # void icu::DateIntervalFormat::setContext(
    #       UDisplayContext value,
    #       UErrorCode &status
    # )
    fmt.set_context(
        UDisplayContext.UDISPCTX_CAPITALIZATION_FOR_MIDDLE_OF_SENTENCE)
    assert (fmt.get_context(UDisplayContextType.UDISPCTX_TYPE_CAPITALIZATION)
            == UDisplayContext.UDISPCTX_CAPITALIZATION_FOR_MIDDLE_OF_SENTENCE)


def test_get_time_zone_upcasting():
    from icupy import BasicTimeZone, SimpleTimeZone

    fmt = DateIntervalFormat.create_instance("yMMMd")

    fmt.set_time_zone(TimeZone.get_gmt())
    zone = fmt.get_time_zone()
    assert isinstance(zone, SimpleTimeZone)

    # TimeZone -> BasicTimeZone
    fmt.set_time_zone(TimeZone.create_time_zone("JST"))
    zone = fmt.get_time_zone()
    assert not isinstance(zone, SimpleTimeZone)
    assert isinstance(zone, BasicTimeZone)



import pytest
from icupy import (
    ConstVoidPtr,
    ICUError, UBlockCode, UCharCategory, UCharDirection, UCharNameChoice,
    UCPMapRangeOption, UCPMapValueFilterPtr,
    UErrorCode, UDecompositionType, UEastAsianWidth, UnicodeSet, UProperty,
    UPropertyNameChoice,
    U_FOLD_CASE_DEFAULT, U_FOLD_CASE_EXCLUDE_SPECIAL_I, U_NO_NUMERIC_VALUE,
    u_char_age, u_char_digit_value, u_char_direction, u_char_from_name,
    u_char_mirror, u_char_name, u_char_type, u_digit, u_fold_case,
    u_for_digit, u_get_bidi_paired_bracket, u_get_binary_property_set,
    u_get_combining_class, u_get_fc_nfkc_closure, u_get_int_property_map,
    u_get_int_property_max_value, u_get_int_property_min_value,
    u_get_int_property_value, u_get_numeric_value, u_get_property_enum,
    u_get_property_name, u_get_property_value_enum,
    u_get_property_value_name, u_get_unicode_version, u_has_binary_property,
    u_isalnum, u_isalpha, u_isbase, u_isblank, u_iscntrl, u_isdefined,
    u_isdigit, u_isgraph, u_is_id_ignorable, u_is_id_part, u_is_id_start,
    u_is_iso_control, u_is_java_id_part, u_is_java_id_start,
    u_is_java_space_char, u_islower, u_is_mirrored, u_isprint, u_ispunct,
    u_isspace, u_istitle, u_is_ualphabetic, u_is_ulowercase, u_isupper,
    u_is_uuppercase, u_is_uwhite_space, u_is_whitespace, u_isxdigit,
    u_tolower, u_totitle, u_toupper, ublock_get_code, ucpmap_get,
    ucpmap_get_range,
)


def test_api():
    # U+FF11: Fullwidth Digit One
    assert u_char_digit_value(ord("1")) == 1
    assert u_char_digit_value(0xff11) == 1

    # U+0631: Arabic Letter Reh
    assert u_char_direction(ord("A")) == UCharDirection.U_LEFT_TO_RIGHT
    assert u_char_direction(0x0631) == UCharDirection.U_RIGHT_TO_LEFT_ARABIC

    # U+1F338: Cherry Blossom
    assert u_char_from_name(UCharNameChoice.U_UNICODE_CHAR_NAME,
                            "Cherry Blossom") == 0x1f338
    assert u_char_from_name(UCharNameChoice.U_EXTENDED_CHAR_NAME,
                            "<Other Symbol-1f338>") == 0x1f338
    with pytest.raises(ICUError) as exc_info:
        _ = u_char_from_name(UCharNameChoice.U_EXTENDED_CHAR_NAME,
                             "<So-1f338>")
    assert exc_info.value.args[0] == UErrorCode.U_ILLEGAL_CHAR_FOUND

    # U+300C: Left Corner Bracket
    # U+300D: Right Corner Bracket
    assert u_char_mirror(ord("(")) == ord(")")
    assert u_char_mirror(0x300d) == 0x300c

    assert u_char_name(
        0x1f338,
        UCharNameChoice.U_UNICODE_CHAR_NAME) == "CHERRY BLOSSOM"

    assert u_char_type(ord("0")) == UCharCategory.U_DECIMAL_DIGIT_NUMBER
    assert u_char_type(ord("(")) == UCharCategory.U_START_PUNCTUATION

    assert u_digit(ord("A"), 16) == 10
    assert u_digit(0xff11, 10) == 1

    # U+0131: Latin Small Letter Dotless I
    assert u_fold_case(ord('I'), U_FOLD_CASE_DEFAULT) == ord("i")
    assert u_fold_case(ord('I'), U_FOLD_CASE_EXCLUDE_SPECIAL_I) == 0x0131

    assert u_for_digit(10, 16) == ord("a")
    assert u_for_digit(10, 10) == 0

    assert u_get_bidi_paired_bracket(ord("[")) == ord("]")
    assert u_get_bidi_paired_bracket(0x300c) == 0x300d

    # Combining Class: Kana Voicing
    # U+3099: Combining Katakana-Hiragana Voiced Sound Mark
    # U+309A: Combining Katakana-Hiragana Semi-Voiced Sound Mark
    assert u_get_combining_class(0x3099) == u_get_combining_class(0x309a) == 8

    # U+2121: Telephone Sign
    assert u_get_fc_nfkc_closure(0x2121) == "tel"

    # UCHAR_IDEOGRAPHIC: 0/1 (false/true)
    assert u_get_int_property_max_value(UProperty.UCHAR_IDEOGRAPHIC) == 1
    assert u_get_int_property_min_value(UProperty.UCHAR_IDEOGRAPHIC) == 0

    assert (u_get_int_property_value(ord("1"),
                                     UProperty.UCHAR_EAST_ASIAN_WIDTH)
            == UEastAsianWidth.U_EA_NARROW)
    assert (u_get_int_property_value(0xff11,
                                     UProperty.UCHAR_EAST_ASIAN_WIDTH)
            == UEastAsianWidth.U_EA_FULLWIDTH)

    # U+5146: CJK Unified Ideograph-5146 (= 10^12)
    # U+4EAC: CJK Unified Ideograph-4EAC (= 10^16)
    assert u_get_numeric_value(ord("1")) == 1.0
    assert u_get_numeric_value(0x5146) == 1000000000000.0
    assert u_get_numeric_value(0x4eac) == U_NO_NUMERIC_VALUE

    assert u_get_property_enum("gcm") == UProperty.UCHAR_GENERAL_CATEGORY_MASK

    assert u_get_property_name(
        UProperty.UCHAR_GENERAL_CATEGORY_MASK,
        UPropertyNameChoice.U_LONG_PROPERTY_NAME) == "General_Category_Mask"

    assert u_get_property_value_enum(
        UProperty.UCHAR_DECOMPOSITION_TYPE,
        "Can") == UDecompositionType.U_DT_CANONICAL

    assert u_get_property_value_name(
        UProperty.UCHAR_DECOMPOSITION_TYPE,
        UDecompositionType.U_DT_CANONICAL,
        UPropertyNameChoice.U_LONG_PROPERTY_NAME) == "Canonical"

    assert u_has_binary_property(0x300c, UProperty.UCHAR_BIDI_MIRRORED)

    assert u_isalnum(ord("a"))
    assert u_isalpha(ord("a"))
    assert u_isbase(ord("a"))
    assert u_isblank(0x3000)  # U+3000: Ideographic Space
    assert u_iscntrl(0x0a)  # U+000A: <End of Line> (EOL, LF, NL)
    assert not u_isdefined(0xfdd0)  # U+FDD0: Undefined Character
    assert u_isdigit(0xff11)  # U+FF11: Fullwidth Digit One
    assert u_isgraph(0x300c)  # U+300C: Left Corner Bracket
    assert u_is_id_ignorable(0xad)  # U+00AD: Soft Hyphen (SHY)
    assert u_is_id_part(0x16ee)  # U+16EE: Runic Arlaug Symbol
    assert u_is_id_start(0x16ee)  # U+16EE: Runic Arlaug Symbol
    assert u_is_iso_control(0x00)  # U+0000: <Null> (NUL)
    assert u_is_java_id_part(ord("$"))  # U+0024: Dollar Sign
    assert u_is_java_id_start(ord("_"))  # U+005F: Low Line
    assert u_is_java_space_char(0x20)  # U+0020: Space (SP)
    assert u_islower(ord("a"))
    assert u_is_mirrored(0x300c)  # U+300C: Left Corner Bracket
    assert u_isprint(ord("a"))
    assert u_ispunct(ord("_"))
    assert u_isspace(0x3000)  # U+3000: Ideographic Space
    # U+01C5: Latin Capital Letter D with Small Letter Z with Caron
    assert u_istitle(0x01c5)
    assert u_is_ualphabetic(ord("A"))
    assert u_is_ulowercase(ord("a"))
    assert u_isupper(ord("A"))
    assert u_is_uuppercase(ord("A"))
    assert u_is_uwhite_space(0x3000)  # U+3000: Ideographic Space
    assert u_is_whitespace(0x3000)  # U+3000: Ideographic Space
    assert u_isxdigit(0xff21)  # U+FF21: Fullwidth Latin Capital Letter A

    # U+FF21: Fullwidth Latin Capital Letter A
    # U+FF41: Fullwidth Latin Small Letter A
    assert u_tolower(0xff21) == 0xff41

    # U+01C5: Latin Capital Letter D with Small Letter Z with Caron
    # U+01C6: Latin Small Letter Dz with Caron
    assert u_totitle(0x01c6) == 0x01c5

    # U+FF21: Fullwidth Latin Capital Letter A
    # U+FF41: Fullwidth Latin Small Letter A
    assert u_toupper(0xff41) == 0xff21

    # U+01C5: Latin Capital Letter D with Small Letter Z with Caron
    assert ublock_get_code(0x01c5) == UBlockCode.UBLOCK_LATIN_EXTENDED_B


def test_u_char_age():
    # void u_charAge(UChar32 c,
    #                UVersionInfo versionArray
    # )
    # U+1F338: Cherry Blossom
    version_array = u_char_age(0x1f338)
    assert isinstance(version_array, list)
    assert len(version_array) == 4
    assert version_array[0] == 6
    assert version_array[1] == 0
    assert version_array[2] == 0
    assert version_array[3] == 0


def test_u_get_binary_property_set():
    # const USet *u_getBinaryPropertySet(UProperty property,
    #                                    UErrorCode *pErrorCode
    # )
    uset = u_get_binary_property_set(UProperty.UCHAR_POSIX_BLANK)
    uniset = UnicodeSet.from_uset(uset)
    assert uniset.size() > 0
    assert uniset.contains(0x20)  # U+0020: Space (SP)
    assert uniset.contains(0xa0)  # U+00A0: No-Break Space (NBSP)
    assert uniset.contains(0x3000)  # U+3000: Ideographic Space


def test_u_get_int_property_map():
    # UCPMap *u_getIntPropertyMap(UProperty property,
    #                             UErrorCode *pErrorCode
    # )
    ucpmap = u_get_int_property_map(UProperty.UCHAR_EAST_ASIAN_WIDTH)

    # U+FF11: Fullwidth Digit One
    assert ucpmap_get(ucpmap, ord("1")) == UEastAsianWidth.U_EA_NARROW
    assert ucpmap_get(ucpmap, 0xff11) == UEastAsianWidth.U_EA_FULLWIDTH

    # uint32_t UCPMapValueFilter(const void *context,
    #                            uint32_t value)
    def _filter1(_context: object, _value: int) -> int:
        assert _context is None
        return _value | 0x1000

    def _filter2(_context: object, _value: int) -> int:
        assert isinstance(_context, dict)
        return _value | _context["key"]

    # UChar32 ucpmap_getRange(const UCPMap *map,
    #                         UChar32 start,
    #                         UCPMapRangeOption option,
    #                         uint32_t surrogateValue,
    #                         UCPMapValueFilter *filter,
    #                         const void *context,
    #                         uint32_t *pValue
    # )
    # Without filter
    result, value = ucpmap_get_range(ucpmap,
                                     0xff11,
                                     UCPMapRangeOption.UCPMAP_RANGE_NORMAL,
                                     0,
                                     UCPMapValueFilterPtr(None),
                                     ConstVoidPtr(None))
    assert result != -1
    assert value == UEastAsianWidth.U_EA_FULLWIDTH

    # With filter
    result, value = ucpmap_get_range(ucpmap,
                                     0xff11,
                                     UCPMapRangeOption.UCPMAP_RANGE_NORMAL,
                                     0,
                                     UCPMapValueFilterPtr(_filter1),
                                     ConstVoidPtr(None))
    assert result != -1
    assert value == UEastAsianWidth.U_EA_FULLWIDTH | 0x1000

    # With filter and context
    context = {"key": 0x2000}
    result, value = ucpmap_get_range(ucpmap,
                                     0xff11,
                                     UCPMapRangeOption.UCPMAP_RANGE_NORMAL,
                                     0,
                                     UCPMapValueFilterPtr(_filter2),
                                     ConstVoidPtr(context))
    assert result != -1
    assert value == UEastAsianWidth.U_EA_FULLWIDTH | 0x2000


def test_u_get_unicode_version():
    # void u_getUnicodeVersion(UVersionInfo versionArray)
    version_array = u_get_unicode_version()
    assert isinstance(version_array, list)
    assert len(version_array) == 4
    assert version_array[0] > 0
    assert version_array[1] >= 0
    assert version_array[2] >= 0
    assert version_array[3] >= 0
